package adapter_test

import (
	"flag"
	"log"
	"net/url"
	"os"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/bundbtest"
	"github.com/ory/dockertest/v3"
	"github.com/pressly/goose/v3"
	"github.com/uptrace/bun"
)

var (
	dbDSN = os.Getenv("TESTS_DB_DSN")
)

func TestMain(m *testing.M) {
	setUpGoose()

	// https://github.com/golang/go/blob/7cfa7d69259590319524c3715df4a39b39924bc3/src/testing/testing.go#L224
	flag.Parse()

	if dbDSN != "" || testing.Short() {
		os.Exit(m.Run())
	}

	pool, err := dockertest.NewPool("")
	if err != nil {
		log.Fatalf("couldn't construct pool: %s", err)
	}

	pool.MaxWait = 30 * time.Second
	if err = pool.Client.Ping(); err != nil {
		log.Fatalf("couldn't connect to Docker: %s", err)
	}

	var resources []*dockertest.Resource

	if dbDSN == "" {
		resource, dsn := newPostgres(pool)
		dbDSN = dsn.String()
		resources = append(resources, resource)
	}

	code := m.Run()

	for _, r := range resources {
		_ = pool.Purge(r)
	}

	os.Exit(code)
}

func setUpGoose() {
	goose.SetBaseFS(os.DirFS("../../migrations"))
	goose.SetLogger(goose.NopLogger())
	if err := goose.SetDialect("postgres"); err != nil {
		log.Fatal(err)
	}
}

func newPostgres(pool *dockertest.Pool) (*dockertest.Resource, *url.URL) {
	resource, dsn, err := bundbtest.NewPostgres(pool, 240)
	if err != nil {
		log.Fatal(err)
	}
	return resource, dsn
}

func newBunDB(tb testing.TB) *bun.DB {
	tb.Helper()
	return bundbtest.NewDB(tb, dbDSN)
}
