package bundbtest

import (
	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/require"
	"github.com/uptrace/bun"
	"github.com/uptrace/bun/dbfixture"
)

var modelsToRegister = []any{
	&bunmodel.Version{},
	&bunmodel.Server{},
	&bunmodel.Tribe{},
	&bunmodel.Player{},
	&bunmodel.Village{},
	&bunmodel.Ennoblement{},
	&bunmodel.PlayerSnapshot{},
	&bunmodel.TribeSnapshot{},
	&bunmodel.TribeChange{},
}

type Fixture struct {
	*dbfixture.Fixture
}

func NewFixture(bunDB *bun.DB) *Fixture {
	// ensure that models are registered
	bunDB.RegisterModel(modelsToRegister...)
	return &Fixture{
		Fixture: dbfixture.New(bunDB),
	}
}

func (f *Fixture) Server(t TestingT, id string) domain.Server {
	t.Helper()

	row, err := f.Row("Server." + id)
	require.NoError(t, err)
	s, ok := row.(*bunmodel.Server)
	require.True(t, ok)
	return s.ToDomain()
}

func (f *Fixture) Tribes(t TestingT, ids ...string) []domain.Tribe {
	t.Helper()

	tribes := make([]domain.Tribe, 0, len(ids))
	for _, id := range ids {
		tribes = append(tribes, f.Tribe(t, id))
	}

	return tribes
}

func (f *Fixture) Tribe(t TestingT, id string) domain.Tribe {
	t.Helper()

	row, err := f.Row("Tribe." + id)
	require.NoError(t, err)
	tribe, ok := row.(*bunmodel.Tribe)
	require.True(t, ok)
	return tribe.ToDomain()
}

func (f *Fixture) Players(t TestingT, ids ...string) []domain.Player {
	t.Helper()

	players := make([]domain.Player, 0, len(ids))
	for _, id := range ids {
		players = append(players, f.Player(t, id))
	}

	return players
}

func (f *Fixture) Player(t TestingT, id string) domain.Player {
	t.Helper()

	row, err := f.Row("Player." + id)
	require.NoError(t, err)
	p, ok := row.(*bunmodel.Player)
	require.True(t, ok)
	return p.ToDomain()
}

func (f *Fixture) Villages(t TestingT, ids ...string) []domain.Village {
	t.Helper()

	villages := make([]domain.Village, 0, len(ids))
	for _, id := range ids {
		villages = append(villages, f.Village(t, id))
	}

	return villages
}

func (f *Fixture) Village(t TestingT, id string) domain.Village {
	t.Helper()

	row, err := f.Row("Village." + id)
	require.NoError(t, err)
	v, ok := row.(*bunmodel.Village)
	require.True(t, ok)
	return v.ToDomain()
}

func (f *Fixture) Ennoblements(t TestingT, ids ...string) []domain.Ennoblement {
	t.Helper()

	ennoblements := make([]domain.Ennoblement, 0, len(ids))
	for _, id := range ids {
		ennoblements = append(ennoblements, f.Ennoblement(t, id))
	}

	return ennoblements
}

func (f *Fixture) Ennoblement(t TestingT, id string) domain.Ennoblement {
	t.Helper()

	row, err := f.Row("Ennoblement." + id)
	require.NoError(t, err)
	e, ok := row.(*bunmodel.Ennoblement)
	require.True(t, ok)
	return e.ToDomain()
}

func (f *Fixture) PlayerSnapshots(t TestingT, ids ...string) []domain.PlayerSnapshot {
	t.Helper()

	snapshots := make([]domain.PlayerSnapshot, 0, len(ids))
	for _, id := range ids {
		snapshots = append(snapshots, f.PlayerSnapshot(t, id))
	}

	return snapshots
}

func (f *Fixture) PlayerSnapshot(t TestingT, id string) domain.PlayerSnapshot {
	t.Helper()

	row, err := f.Row("PlayerSnapshot." + id)
	require.NoError(t, err)
	ps, ok := row.(*bunmodel.PlayerSnapshot)
	require.True(t, ok)
	return ps.ToDomain()
}

func (f *Fixture) TribeSnapshots(t TestingT, ids ...string) []domain.TribeSnapshot {
	t.Helper()

	snapshots := make([]domain.TribeSnapshot, 0, len(ids))
	for _, id := range ids {
		snapshots = append(snapshots, f.TribeSnapshot(t, id))
	}

	return snapshots
}

func (f *Fixture) TribeSnapshot(t TestingT, id string) domain.TribeSnapshot {
	t.Helper()

	row, err := f.Row("TribeSnapshot." + id)
	require.NoError(t, err)
	ts, ok := row.(*bunmodel.TribeSnapshot)
	require.True(t, ok)
	return ts.ToDomain()
}

func (f *Fixture) TribeChanges(t TestingT, ids ...string) []domain.TribeChange {
	t.Helper()

	tcs := make([]domain.TribeChange, 0, len(ids))
	for _, id := range ids {
		tcs = append(tcs, f.TribeChange(t, id))
	}

	return tcs
}

func (f *Fixture) TribeChange(t TestingT, id string) domain.TribeChange {
	t.Helper()

	row, err := f.Row("TribeChange." + id)
	require.NoError(t, err)
	tc, ok := row.(*bunmodel.TribeChange)
	require.True(t, ok)
	return tc.ToDomain()
}
