package bundbtest

import (
	"fmt"
	"net"
	"net/url"
	"os"

	"github.com/ory/dockertest/v3"
	"github.com/ory/dockertest/v3/docker"
)

func NewPostgres(pool *dockertest.Pool, expSeconds uint) (*dockertest.Resource, *url.URL, error) {
	q := url.Values{}
	q.Add("sslmode", "disable")
	dsn := &url.URL{
		Scheme:   "postgres",
		User:     url.UserPassword("postgres", "postgres"),
		Path:     "twhelpdb",
		RawQuery: q.Encode(),
	}

	pw, _ := dsn.User.Password()

	resource, err := pool.RunWithOptions(&dockertest.RunOptions{
		Repository: "postgres",
		Tag:        "14.9",
		Env: []string{
			fmt.Sprintf("POSTGRES_USER=%s", dsn.User.Username()),
			fmt.Sprintf("POSTGRES_PASSWORD=%s", pw),
			fmt.Sprintf("POSTGRES_DB=%s", dsn.Path),
		},
	}, func(config *docker.HostConfig) {
		config.AutoRemove = true
		config.RestartPolicy = docker.RestartPolicy{
			Name: "no",
		}
	})
	if err != nil {
		return nil, nil, fmt.Errorf("couldn't run postgres: %w", err)
	}

	if err = resource.Expire(expSeconds); err != nil {
		return nil, nil, err
	}

	dsn.Host, err = getHostPort(resource, "5432/tcp")
	if err != nil {
		return nil, nil, err
	}

	return resource, dsn, nil
}

func getHostPort(resource *dockertest.Resource, id string) (string, error) {
	dockerURL := os.Getenv("DOCKER_HOST")
	if dockerURL == "" {
		return resource.GetHostPort(id), nil
	}

	u, err := url.Parse(dockerURL)
	if err != nil {
		return "", err
	}

	return net.JoinHostPort(u.Hostname(), resource.GetPort(id)), nil
}
