package adapter

import (
	"context"
	"fmt"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/publishermodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/ThreeDotsLabs/watermill/message"
)

type EnnoblementWatermillPublisher struct {
	publisher message.Publisher
	marshaler WatermillMarshaler
}

func NewEnnoblementWatermillPublisher(publisher message.Publisher, marshaler WatermillMarshaler) *EnnoblementWatermillPublisher {
	return &EnnoblementWatermillPublisher{publisher: publisher, marshaler: marshaler}
}

func (e *EnnoblementWatermillPublisher) CmdRefresh(ctx context.Context, payloads ...domain.RefreshEnnoblementsCmdPayload) error {
	msgs := make([]*message.Message, 0, len(payloads))
	for _, p := range payloads {
		msg, err := e.marshaler.Marshal(publishermodel.RefreshEnnoblementsCmdPayload(p))
		if err != nil {
			return fmt.Errorf("%s: couldn't marshal RefreshEnnoblementsCmdPayload: %w", p.Key, err)
		}

		msg.SetContext(ctx)

		msgs = append(msgs, msg)
	}

	topic := "ennoblements.cmd.refresh"
	if err := e.publisher.Publish(topic, msgs...); err != nil {
		return fmt.Errorf("couldn't publish message with topic %s: %w", topic, err)
	}

	return nil
}
