package adapter_test

import (
	"context"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter"
	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/publishermodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/ThreeDotsLabs/watermill"
	"github.com/ThreeDotsLabs/watermill/message/subscriber"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestEnnoblementWatermillPublisher_CmdRefresh(t *testing.T) {
	t.Parallel()

	marshaler := adapter.JSONWatermillMarshaler{
		NewUUID: watermill.NewUUID,
	}
	pubSub := newPubSub(t)

	payloads := []domain.RefreshEnnoblementsCmdPayload{
		{
			Key:         "pl151",
			URL:         "https://pl151.plemiona.pl",
			VersionCode: "pl",
		},
		{
			Key:         "en113",
			URL:         "https://en113.tribalwars.net",
			VersionCode: "en",
		},
	}

	msgs, err := pubSub.Subscribe(context.Background(), "ennoblements.cmd.refresh")
	require.NoError(t, err)

	require.NoError(t, adapter.NewEnnoblementWatermillPublisher(pubSub, marshaler).CmdRefresh(context.Background(), payloads...))

	receivedMsgs, _ := subscriber.BulkRead(msgs, len(payloads), time.Second)
	require.Len(t, receivedMsgs, len(payloads))
	for _, m := range receivedMsgs {
		var received publishermodel.RefreshEnnoblementsCmdPayload
		assert.NoError(t, marshaler.Unmarshal(m, &received))
		assert.Contains(t, payloads, domain.RefreshEnnoblementsCmdPayload{
			Key:         received.Key,
			URL:         received.URL,
			VersionCode: received.VersionCode,
		})
	}
}
