package bunmodel

import "gitea.dwysokinski.me/twhelp/core/internal/domain"

type Building struct {
	MaxLevel        int8
	MinLevel        int8
	Wood            int32
	Stone           int32
	Iron            int32
	Pop             int16
	WoodFactor      float64
	StoneFactor     float64
	IronFactor      float64
	PopFactor       float64
	BuildTime       float64
	BuildTimeFactor float64
}

type BuildingInfo struct {
	Main       Building
	Barracks   Building
	Stable     Building
	Garage     Building
	Watchtower Building
	Snob       Building
	Smith      Building
	Place      Building
	Statue     Building
	Market     Building
	Wood       Building
	Stone      Building
	Iron       Building
	Farm       Building
	Storage    Building
	Hide       Building
	Wall       Building
}

func NewBuildingInfo(info domain.BuildingInfo) BuildingInfo {
	return BuildingInfo{
		Main:       Building(info.Main),
		Barracks:   Building(info.Barracks),
		Stable:     Building(info.Stable),
		Garage:     Building(info.Garage),
		Watchtower: Building(info.Watchtower),
		Snob:       Building(info.Snob),
		Smith:      Building(info.Smith),
		Place:      Building(info.Place),
		Statue:     Building(info.Statue),
		Market:     Building(info.Market),
		Wood:       Building(info.Wood),
		Stone:      Building(info.Stone),
		Iron:       Building(info.Iron),
		Farm:       Building(info.Farm),
		Storage:    Building(info.Storage),
		Hide:       Building(info.Hide),
		Wall:       Building(info.Wall),
	}
}

func (b BuildingInfo) ToDomain() domain.BuildingInfo {
	return domain.BuildingInfo{
		Main:       domain.Building(b.Main),
		Barracks:   domain.Building(b.Barracks),
		Stable:     domain.Building(b.Stable),
		Garage:     domain.Building(b.Garage),
		Watchtower: domain.Building(b.Watchtower),
		Snob:       domain.Building(b.Snob),
		Smith:      domain.Building(b.Smith),
		Place:      domain.Building(b.Place),
		Statue:     domain.Building(b.Statue),
		Market:     domain.Building(b.Market),
		Wood:       domain.Building(b.Wood),
		Stone:      domain.Building(b.Stone),
		Iron:       domain.Building(b.Iron),
		Farm:       domain.Building(b.Farm),
		Storage:    domain.Building(b.Storage),
		Hide:       domain.Building(b.Hide),
		Wall:       domain.Building(b.Wall),
	}
}
