package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"github.com/uptrace/bun"
)

type Ennoblement struct {
	bun.BaseModel `bun:"table:ennoblements,alias:ennoblement"`

	ID         int64     `bun:"id,pk,autoincrement,identity"`
	ServerKey  string    `bun:"server_key,nullzero"`
	VillageID  int64     `bun:"village_id,nullzero"`
	Village    Village   `bun:"village,rel:belongs-to,join:village_id=id,join:server_key=server_key"`
	NewOwnerID int64     `bun:"new_owner_id,nullzero"`
	NewOwner   Player    `bun:"new_owner,rel:belongs-to,join:new_owner_id=id,join:server_key=server_key"`
	NewTribeID int64     `bun:"new_tribe_id,nullzero"`
	NewTribe   Tribe     `bun:"new_tribe,rel:belongs-to,join:new_tribe_id=id,join:server_key=server_key"`
	OldOwnerID int64     `bun:"old_owner_id,nullzero"`
	OldOwner   Player    `bun:"old_owner,rel:belongs-to,join:old_owner_id=id,join:server_key=server_key"`
	OldTribeID int64     `bun:"old_tribe_id,nullzero"`
	OldTribe   Tribe     `bun:"old_tribe,rel:belongs-to,join:old_tribe_id=id,join:server_key=server_key"`
	Points     int64     `bun:"points"`
	CreatedAt  time.Time `bun:"created_at,nullzero"`
}

func NewEnnoblement(p domain.CreateEnnoblementParams) Ennoblement {
	return Ennoblement{
		ServerKey:  p.ServerKey,
		VillageID:  p.VillageID,
		NewOwnerID: p.NewOwnerID,
		NewTribeID: p.NewTribeID,
		OldOwnerID: p.OldOwnerID,
		OldTribeID: p.OldTribeID,
		Points:     p.Points,
		CreatedAt:  p.CreatedAt,
	}
}

func (e Ennoblement) ToDomain() domain.Ennoblement {
	return domain.Ennoblement{
		BaseEnnoblement: domain.BaseEnnoblement{
			VillageID:  e.VillageID,
			NewOwnerID: e.NewOwnerID,
			NewTribeID: e.NewTribeID,
			OldOwnerID: e.OldOwnerID,
			OldTribeID: e.OldTribeID,
			Points:     e.Points,
			CreatedAt:  e.CreatedAt,
		},
		ID:        e.ID,
		ServerKey: e.ServerKey,
	}
}

func (e Ennoblement) ToDomainWithRelations() domain.EnnoblementWithRelations {
	res := domain.EnnoblementWithRelations{
		Ennoblement: e.ToDomain(),
		Village:     e.Village.ToMeta(),
	}

	if e.NewOwner.ID > 0 {
		res.NewOwner = domain.NullPlayerMeta{
			Valid:  true,
			Player: e.NewOwner.ToMeta(),
		}
	}
	if e.NewTribe.ID > 0 {
		res.NewTribe = domain.NullTribeMeta{
			Valid: true,
			Tribe: e.NewTribe.ToMeta(),
		}
	}
	if e.OldOwner.ID > 0 {
		res.OldOwner = domain.NullPlayerMeta{
			Valid:  true,
			Player: e.OldOwner.ToMeta(),
		}
	}
	if e.OldTribe.ID > 0 {
		res.OldTribe = domain.NullTribeMeta{
			Valid: true,
			Tribe: e.OldTribe.ToMeta(),
		}
	}

	return res
}
