package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestEnnoblement(t *testing.T) {
	t.Parallel()

	var id int64 = 1234
	params := domain.CreateEnnoblementParams{
		BaseEnnoblement: domain.BaseEnnoblement{
			VillageID:  112,
			NewOwnerID: 113,
			NewTribeID: 114,
			OldOwnerID: 115,
			OldTribeID: 116,
			Points:     117,
			CreatedAt:  time.Now().Add(-20 * time.Hour),
		},
		ServerKey: "pl151",
	}

	result := bunmodel.NewEnnoblement(params)
	result.ID = id

	ennoblement := result.ToDomain()
	assert.Equal(t, params.VillageID, ennoblement.VillageID)
	assert.Equal(t, params.NewOwnerID, ennoblement.NewOwnerID)
	assert.Equal(t, params.NewTribeID, ennoblement.NewTribeID)
	assert.Equal(t, params.OldOwnerID, ennoblement.OldOwnerID)
	assert.Equal(t, params.OldTribeID, ennoblement.OldTribeID)
	assert.Equal(t, params.Points, ennoblement.Points)
	assert.Equal(t, params.CreatedAt, ennoblement.CreatedAt)
	assert.Equal(t, params.ServerKey, ennoblement.ServerKey)
	assert.Equal(t, id, ennoblement.ID)

	result.Village = bunmodel.Village{
		ID: 12345,
	}
	result.NewOwner = bunmodel.Player{
		ID: 11231,
	}
	result.NewTribe = bunmodel.Tribe{
		ID: 11232,
	}
	result.OldOwner = bunmodel.Player{
		ID: 11233,
	}
	result.OldTribe = bunmodel.Tribe{
		ID: 11234,
	}

	ennoblementWithRelations := result.ToDomainWithRelations()
	assert.Equal(t, ennoblement, ennoblementWithRelations.Ennoblement)
	assert.Equal(t, result.Village.ID, ennoblementWithRelations.Village.ID)
	assert.True(t, ennoblementWithRelations.NewOwner.Valid)
	assert.Equal(t, result.NewOwner.ID, ennoblementWithRelations.NewOwner.Player.ID)
	assert.True(t, ennoblementWithRelations.NewTribe.Valid)
	assert.Equal(t, result.NewTribe.ID, ennoblementWithRelations.NewTribe.Tribe.ID)
	assert.True(t, ennoblementWithRelations.OldOwner.Valid)
	assert.Equal(t, result.OldOwner.ID, ennoblementWithRelations.OldOwner.Player.ID)
	assert.True(t, ennoblementWithRelations.OldTribe.Valid)
	assert.Equal(t, result.OldTribe.ID, ennoblementWithRelations.OldTribe.Tribe.ID)
}
