package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/uptrace/bun"
)

type PlayerSnapshot struct {
	bun.BaseModel `bun:"table:player_snapshots,alias:ps"`

	ID          int64     `bun:"id,pk,autoincrement,identity"`
	PlayerID    int64     `bun:"player_id,nullzero"`
	NumVillages int64     `bun:"num_villages"`
	Points      int64     `bun:"points"`
	Rank        int64     `bun:"rank"`
	TribeID     int64     `bun:"tribe_id,nullzero"`
	Tribe       Tribe     `bun:"tribe,rel:belongs-to,join:tribe_id=id,join:server_key=server_key"`
	ServerKey   string    `bun:"server_key,nullzero"`
	Date        time.Time `bun:"date,nullzero"`
	CreatedAt   time.Time `bun:"created_at,nullzero"`

	OpponentsDefeated
}

func NewPlayerSnapshot(p domain.CreatePlayerSnapshotParams) PlayerSnapshot {
	return PlayerSnapshot{
		OpponentsDefeated: OpponentsDefeated(p.OpponentsDefeated),
		PlayerID:          p.PlayerID,
		NumVillages:       p.NumVillages,
		Points:            p.Points,
		Rank:              p.Rank,
		TribeID:           p.TribeID,
		ServerKey:         p.ServerKey,
		Date:              p.Date,
		CreatedAt:         time.Now(),
	}
}

func (p PlayerSnapshot) ToDomain() domain.PlayerSnapshot {
	return domain.PlayerSnapshot{
		OpponentsDefeated: domain.OpponentsDefeated(p.OpponentsDefeated),
		ID:                p.ID,
		PlayerID:          p.PlayerID,
		NumVillages:       p.NumVillages,
		Points:            p.Points,
		Rank:              p.Rank,
		TribeID:           p.TribeID,
		ServerKey:         p.ServerKey,
		Date:              p.Date,
		CreatedAt:         p.CreatedAt,
	}
}

func (p PlayerSnapshot) ToDomainWithRelations() domain.PlayerSnapshotWithRelations {
	res := domain.PlayerSnapshotWithRelations{
		PlayerSnapshot: p.ToDomain(),
	}

	if p.Tribe.ID > 0 {
		res.Tribe = domain.NullTribeMeta{
			Valid: true,
			Tribe: p.Tribe.ToMeta(),
		}
	}

	return res
}
