package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestPlayerSnapshot(t *testing.T) {
	t.Parallel()

	var id int64 = 12342
	params := domain.CreatePlayerSnapshotParams{
		OpponentsDefeated: domain.OpponentsDefeated{
			RankAtt:    1,
			ScoreAtt:   2,
			RankDef:    3,
			ScoreDef:   4,
			RankSup:    5,
			ScoreSup:   6,
			RankTotal:  7,
			ScoreTotal: 8,
		},
		PlayerID:    1234,
		NumVillages: 111,
		Points:      112,
		Rank:        113,
		TribeID:     114,
		ServerKey:   "pl151",
		Date:        time.Date(2022, time.December, 24, 0, 0, 0, 0, time.UTC),
	}

	result := bunmodel.NewPlayerSnapshot(params)
	result.ID = id

	snapshot := result.ToDomain()

	assert.EqualValues(t, params.OpponentsDefeated, snapshot.OpponentsDefeated)
	assert.Equal(t, id, snapshot.ID)
	assert.Equal(t, params.PlayerID, snapshot.PlayerID)
	assert.Equal(t, params.NumVillages, snapshot.NumVillages)
	assert.Equal(t, params.Points, snapshot.Points)
	assert.Equal(t, params.Rank, snapshot.Rank)
	assert.Equal(t, params.TribeID, snapshot.TribeID)
	assert.Equal(t, params.ServerKey, snapshot.ServerKey)
	assert.Equal(t, params.Date, snapshot.Date)
	assert.WithinDuration(t, time.Now(), snapshot.CreatedAt, time.Second)

	result.Tribe = bunmodel.Tribe{
		ID: params.TribeID,
	}

	snapshotWithRelations := result.ToDomainWithRelations()
	assert.Equal(t, snapshot, snapshotWithRelations.PlayerSnapshot)
	assert.WithinDuration(t, time.Now(), snapshotWithRelations.CreatedAt, time.Second)
	assert.True(t, snapshotWithRelations.Tribe.Valid)
	assert.Equal(t, params.TribeID, snapshotWithRelations.Tribe.Tribe.ID)
}
