package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestPlayer(t *testing.T) {
	t.Parallel()

	deletedAt := time.Now().Add(-100 * time.Hour)
	params := domain.CreatePlayerParams{
		BasePlayer: domain.BasePlayer{
			OpponentsDefeated: domain.OpponentsDefeated{
				RankAtt:    1,
				ScoreAtt:   2,
				RankDef:    3,
				ScoreDef:   4,
				RankSup:    5,
				ScoreSup:   6,
				RankTotal:  7,
				ScoreTotal: 8,
			},
			ID:          999,
			Name:        "name 999",
			NumVillages: 3,
			Points:      4,
			Rank:        6,
			TribeID:     1234,
			ProfileURL:  "profile-999",
		},
		BestRank:       111,
		BestRankAt:     time.Now().Add(-5 * time.Minute),
		MostPoints:     222,
		MostPointsAt:   time.Now().Add(-1 * time.Minute),
		MostVillages:   333,
		MostVillagesAt: time.Now().Add(-12 * time.Minute),
		LastActivityAt: time.Now().Add(-15 * time.Minute),
		ServerKey:      "pl151",
	}

	result := bunmodel.NewPlayer(params)
	result.DeletedAt = deletedAt

	player := result.ToDomain()
	assert.EqualValues(t, params.OpponentsDefeated, player.OpponentsDefeated)
	assert.Equal(t, params.ID, player.ID)
	assert.Equal(t, params.Name, player.Name)
	assert.Equal(t, params.NumVillages, player.NumVillages)
	assert.Equal(t, params.Points, player.Points)
	assert.Equal(t, params.Rank, player.Rank)
	assert.Equal(t, params.TribeID, player.TribeID)
	assert.Equal(t, params.ProfileURL, player.ProfileURL)
	assert.Equal(t, params.BestRank, player.BestRank)
	assert.Equal(t, params.BestRankAt, player.BestRankAt)
	assert.Equal(t, params.MostPoints, player.MostPoints)
	assert.Equal(t, params.MostPointsAt, player.MostPointsAt)
	assert.Equal(t, params.MostVillages, player.MostVillages)
	assert.Equal(t, params.MostVillagesAt, player.MostVillagesAt)
	assert.Equal(t, params.LastActivityAt, player.LastActivityAt)
	assert.Equal(t, params.ServerKey, player.ServerKey)
	assert.WithinDuration(t, time.Now(), player.CreatedAt, time.Second)
	assert.Equal(t, deletedAt, player.DeletedAt)

	meta := result.ToMeta()
	assert.Equal(t, params.ID, meta.ID)
	assert.Equal(t, params.Name, meta.Name)
	assert.Equal(t, params.ProfileURL, meta.ProfileURL)

	result.Tribe = bunmodel.Tribe{
		ID: params.TribeID,
	}
	result.Server = bunmodel.Server{
		Key: params.ServerKey,
	}

	playerWithRelations := result.ToDomainWithRelations()
	assert.Equal(t, player, playerWithRelations.Player)
	assert.WithinDuration(t, time.Now(), playerWithRelations.CreatedAt, time.Second)
	assert.True(t, playerWithRelations.Tribe.Valid)
	assert.Equal(t, params.TribeID, playerWithRelations.Tribe.Tribe.ID)
	assert.Equal(t, params.ServerKey, playerWithRelations.Server.Key)

	metaWithRelations := result.ToMetaWithRelations()
	assert.Equal(t, meta, metaWithRelations.PlayerMeta)
	assert.True(t, metaWithRelations.Tribe.Valid)
}
