package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"github.com/uptrace/bun"
)

type Server struct {
	bun.BaseModel            `bun:"table:servers,alias:server"`
	Key                      string       `bun:"key,nullzero,pk"`
	URL                      string       `bun:"url,nullzero"`
	Open                     bool         `bun:"open"`
	Special                  bool         `bun:"special"`
	NumPlayers               int64        `bun:"num_players"`
	NumTribes                int64        `bun:"num_tribes"`
	NumVillages              int64        `bun:"num_villages"`
	NumPlayerVillages        int64        `bun:"num_player_villages"`
	NumBarbarianVillages     int64        `bun:"num_barbarian_villages"`
	NumBonusVillages         int64        `bun:"num_bonus_villages"`
	Config                   ServerConfig `bun:"config"`
	BuildingInfo             BuildingInfo `bun:"building_info"`
	UnitInfo                 UnitInfo     `bun:"unit_info"`
	CreatedAt                time.Time    `bun:"created_at,nullzero"`
	PlayerDataUpdatedAt      time.Time    `bun:"player_data_updated_at,nullzero"`
	PlayerSnapshotsCreatedAt time.Time    `bun:"player_snapshots_created_at,nullzero"`
	TribeDataUpdatedAt       time.Time    `bun:"tribe_data_updated_at,nullzero"`
	TribeSnapshotsCreatedAt  time.Time    `bun:"tribe_snapshots_created_at,nullzero"`
	VillageDataUpdatedAt     time.Time    `bun:"village_data_updated_at,nullzero"`
	EnnoblementDataUpdatedAt time.Time    `bun:"ennoblement_data_updated_at,nullzero"`
	VersionCode              string       `bun:"version_code,nullzero"`
}

func (s Server) ToDomain() domain.Server {
	return domain.Server{
		Key:                      s.Key,
		URL:                      s.URL,
		Open:                     s.Open,
		Special:                  s.Special,
		NumPlayers:               s.NumPlayers,
		NumTribes:                s.NumTribes,
		NumVillages:              s.NumVillages,
		NumPlayerVillages:        s.NumPlayerVillages,
		NumBarbarianVillages:     s.NumBarbarianVillages,
		NumBonusVillages:         s.NumBonusVillages,
		Config:                   s.Config.ToDomain(),
		BuildingInfo:             s.BuildingInfo.ToDomain(),
		UnitInfo:                 s.UnitInfo.ToDomain(),
		CreatedAt:                s.CreatedAt,
		PlayerDataUpdatedAt:      s.PlayerDataUpdatedAt,
		PlayerSnapshotsCreatedAt: s.PlayerSnapshotsCreatedAt,
		TribeDataUpdatedAt:       s.TribeDataUpdatedAt,
		TribeSnapshotsCreatedAt:  s.TribeSnapshotsCreatedAt,
		VillageDataUpdatedAt:     s.VillageDataUpdatedAt,
		EnnoblementDataUpdatedAt: s.EnnoblementDataUpdatedAt,
		VersionCode:              s.VersionCode,
	}
}

func (s Server) ToMeta() domain.ServerMeta {
	return domain.ServerMeta{
		Key:  s.Key,
		URL:  s.URL,
		Open: s.Open,
	}
}
