package bunmodel

import "gitea.dwysokinski.me/twhelp/core/internal/domain"

type ServerConfigBuild struct {
	Destroy int8
}

type ServerConfigMisc struct {
	KillRanking     int8
	Tutorial        int8
	TradeCancelTime int16
}

type ServerConfigCommands struct {
	MillisArrival     int16
	CommandCancelTime int16
}

type ServerConfigNewbie struct {
	Days                 int16
	RatioDays            int16
	Ratio                int16
	RemoveNewbieVillages int8
}

type ServerConfigGame struct {
	BuildtimeFormula   int8
	Knight             int8
	KnightNewItems     int8
	Archer             int8
	Tech               int8
	FarmLimit          int32
	Church             int8
	Watchtower         int8
	Stronghold         int8
	FakeLimit          float64
	BarbarianRise      float64
	BarbarianShrink    int8
	BarbarianMaxPoints int32
	Scavenging         int8
	Hauls              int8
	HaulsBase          int32
	HaulsMax           int32
	BaseProduction     int32
	Event              int16
	SuppressEvents     int8
}

type ServerConfigBuildings struct {
	CustomMain       int8
	CustomFarm       int8
	CustomStorage    int8
	CustomPlace      int8
	CustomBarracks   int8
	CustomChurch     int8
	CustomSmith      int8
	CustomWood       int8
	CustomStone      int8
	CustomIron       int8
	CustomMarket     int8
	CustomStable     int8
	CustomWall       int8
	CustomGarage     int8
	CustomHide       int8
	CustomSnob       int8
	CustomStatue     int8
	CustomWatchtower int8
}

type ServerConfigSnob struct {
	Gold          int8
	CheapRebuild  int8
	Rise          int8
	MaxDist       int16
	Factor        float64
	CoinWood      int32
	CoinStone     int32
	CoinIron      int32
	NoBarbConquer int8
}

type ServerConfigAlly struct {
	NoHarm                int8
	NoOtherSupport        int8
	NoOtherSupportType    int8
	AllytimeSupport       int32
	NoLeave               int8
	NoJoin                int8
	Limit                 int16
	FixedAllies           int8
	PointsMemberCount     int32
	WarsMemberRequirement int16
	WarsPointsRequirement int32
	WarsAutoacceptDays    int16
	Levels                int8
	XpRequirements        string
}

type ServerConfigCoord struct {
	MapSize         int16
	Func            int8
	EmptyVillages   int16
	BonusVillages   int16
	BonusNew        int16
	Inner           int32
	SelectStart     int8
	VillageMoveWait int32
	NobleRestart    int8
	StartVillages   int8
}

type ServerConfigSitter struct {
	Allow int8
}

type ServerConfigSleep struct {
	Active   int8
	Delay    int32
	Min      int16
	Max      int16
	MinAwake int16
	MaxAwake int16
	WarnTime int16
}

type ServerConfigNight struct {
	Active    int8
	StartHour int8
	EndHour   int8
	DefFactor float64
	Duration  int8
}

type ServerConfigWin struct {
	Check int16
}

type ServerConfig struct {
	Speed     float64
	UnitSpeed float64
	Moral     int16
	Build     ServerConfigBuild
	Misc      ServerConfigMisc
	Commands  ServerConfigCommands
	Newbie    ServerConfigNewbie
	Game      ServerConfigGame
	Buildings ServerConfigBuildings
	Snob      ServerConfigSnob
	Ally      ServerConfigAlly
	Coord     ServerConfigCoord
	Sitter    ServerConfigSitter
	Sleep     ServerConfigSleep
	Night     ServerConfigNight
	Win       ServerConfigWin
}

func NewServerConfig(cfg domain.ServerConfig) ServerConfig {
	return ServerConfig{
		Speed:     cfg.Speed,
		UnitSpeed: cfg.UnitSpeed,
		Moral:     cfg.Moral,
		Build:     ServerConfigBuild(cfg.Build),
		Misc:      ServerConfigMisc(cfg.Misc),
		Commands:  ServerConfigCommands(cfg.Commands),
		Newbie:    ServerConfigNewbie(cfg.Newbie),
		Game:      ServerConfigGame(cfg.Game),
		Buildings: ServerConfigBuildings(cfg.Buildings),
		Snob:      ServerConfigSnob(cfg.Snob),
		Ally:      ServerConfigAlly(cfg.Ally),
		Coord:     ServerConfigCoord(cfg.Coord),
		Sitter:    ServerConfigSitter(cfg.Sitter),
		Sleep:     ServerConfigSleep(cfg.Sleep),
		Night:     ServerConfigNight(cfg.Night),
		Win:       ServerConfigWin(cfg.Win),
	}
}

func (s ServerConfig) ToDomain() domain.ServerConfig {
	return domain.ServerConfig{
		Speed:     s.Speed,
		UnitSpeed: s.UnitSpeed,
		Moral:     s.Moral,
		Build:     domain.ServerConfigBuild(s.Build),
		Misc:      domain.ServerConfigMisc(s.Misc),
		Commands:  domain.ServerConfigCommands(s.Commands),
		Newbie:    domain.ServerConfigNewbie(s.Newbie),
		Game:      domain.ServerConfigGame(s.Game),
		Buildings: domain.ServerConfigBuildings(s.Buildings),
		Snob:      domain.ServerConfigSnob(s.Snob),
		Ally:      domain.ServerConfigAlly(s.Ally),
		Coord:     domain.ServerConfigCoord(s.Coord),
		Sitter:    domain.ServerConfigSitter(s.Sitter),
		Sleep:     domain.ServerConfigSleep(s.Sleep),
		Night:     domain.ServerConfigNight(s.Night),
		Win:       domain.ServerConfigWin(s.Win),
	}
}
