package bunmodel_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestServerConfig(t *testing.T) {
	t.Parallel()

	cfg := domain.ServerConfig{
		Speed:     2,
		UnitSpeed: 0.5,
		Moral:     1,
		Build: domain.ServerConfigBuild{
			Destroy: 1,
		},
		Misc: domain.ServerConfigMisc{
			KillRanking:     1,
			Tutorial:        2,
			TradeCancelTime: 3,
		},
		Commands: domain.ServerConfigCommands{
			MillisArrival:     1,
			CommandCancelTime: 2,
		},
		Newbie: domain.ServerConfigNewbie{
			Days:                 1,
			RatioDays:            2,
			Ratio:                3,
			RemoveNewbieVillages: 4,
		},
		Game: domain.ServerConfigGame{
			BuildtimeFormula:   1,
			Knight:             2,
			KnightNewItems:     3,
			Archer:             4,
			Tech:               5,
			FarmLimit:          6,
			Church:             7,
			Watchtower:         8,
			Stronghold:         9,
			FakeLimit:          10,
			BarbarianRise:      11,
			BarbarianShrink:    12,
			BarbarianMaxPoints: 13,
			Scavenging:         20,
			Hauls:              14,
			HaulsBase:          15,
			HaulsMax:           16,
			BaseProduction:     17,
			Event:              18,
			SuppressEvents:     19,
		},
		Buildings: domain.ServerConfigBuildings{
			CustomMain:       -1,
			CustomFarm:       10,
			CustomStorage:    11,
			CustomPlace:      12,
			CustomBarracks:   13,
			CustomChurch:     14,
			CustomSmith:      15,
			CustomWood:       16,
			CustomStone:      17,
			CustomIron:       18,
			CustomMarket:     19,
			CustomStable:     20,
			CustomWall:       21,
			CustomGarage:     22,
			CustomHide:       23,
			CustomSnob:       24,
			CustomStatue:     25,
			CustomWatchtower: 26,
		},
		Snob: domain.ServerConfigSnob{
			Gold:          100,
			CheapRebuild:  101,
			Rise:          102,
			MaxDist:       102,
			Factor:        103,
			CoinWood:      104,
			CoinStone:     105,
			CoinIron:      106,
			NoBarbConquer: 107,
		},
		Ally: domain.ServerConfigAlly{
			NoHarm:                60,
			NoOtherSupport:        61,
			NoOtherSupportType:    62,
			AllytimeSupport:       63,
			NoLeave:               64,
			NoJoin:                65,
			Limit:                 66,
			FixedAllies:           67,
			PointsMemberCount:     68,
			WarsMemberRequirement: 69,
			WarsPointsRequirement: 70,
			WarsAutoacceptDays:    71,
			Levels:                72,
			XpRequirements:        "v1",
		},
		Coord: domain.ServerConfigCoord{
			MapSize:         1000,
			Func:            1,
			EmptyVillages:   2,
			BonusVillages:   3,
			BonusNew:        4,
			Inner:           5,
			SelectStart:     6,
			VillageMoveWait: 7,
			NobleRestart:    8,
			StartVillages:   9,
		},
		Sitter: domain.ServerConfigSitter{
			Allow: 1,
		},
		Sleep: domain.ServerConfigSleep{
			Active:   50,
			Delay:    51,
			Min:      52,
			Max:      53,
			MinAwake: 54,
			MaxAwake: 55,
			WarnTime: 56,
		},
		Night: domain.ServerConfigNight{
			Active:    1,
			StartHour: 23,
			EndHour:   8,
			DefFactor: 1.5,
			Duration:  11,
		},
		Win: domain.ServerConfigWin{
			Check: 16,
		},
	}
	assert.Equal(t, cfg, bunmodel.NewServerConfig(cfg).ToDomain())
}
