package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestServer_ToDomain(t *testing.T) {
	t.Parallel()

	domainModel := domain.Server{
		Key:                  "pl151",
		URL:                  "https://pl151.plemiona.pl",
		Open:                 true,
		Special:              false,
		NumPlayers:           1234,
		NumTribes:            1235,
		NumVillages:          1236,
		NumPlayerVillages:    1111,
		NumBarbarianVillages: 1112,
		NumBonusVillages:     1113,
		Config: domain.ServerConfig{
			Speed: 0.12223,
		},
		BuildingInfo: domain.BuildingInfo{
			Market: domain.Building{BuildTime: 123.333},
		},
		UnitInfo: domain.UnitInfo{
			Spy: domain.Unit{Speed: 6.000001},
		},
		CreatedAt:                time.Now().Add(-20 * time.Hour),
		PlayerDataUpdatedAt:      time.Now().Add(-11 * time.Hour),
		PlayerSnapshotsCreatedAt: time.Now().Add(-1 * time.Hour),
		TribeDataUpdatedAt:       time.Now(),
		TribeSnapshotsCreatedAt:  time.Now().Add(-111 * time.Hour),
		VillageDataUpdatedAt:     time.Now().Add(-5 * time.Hour),
		EnnoblementDataUpdatedAt: time.Now().Add(15 * time.Hour),
		VersionCode:              "pl",
	}
	dbModel := bunmodel.Server{
		Key:                      domainModel.Key,
		URL:                      domainModel.URL,
		Open:                     domainModel.Open,
		Special:                  domainModel.Special,
		NumPlayers:               domainModel.NumPlayers,
		NumTribes:                domainModel.NumTribes,
		NumVillages:              domainModel.NumVillages,
		NumPlayerVillages:        domainModel.NumPlayerVillages,
		NumBarbarianVillages:     domainModel.NumBarbarianVillages,
		NumBonusVillages:         domainModel.NumBonusVillages,
		Config:                   bunmodel.NewServerConfig(domainModel.Config),
		BuildingInfo:             bunmodel.NewBuildingInfo(domainModel.BuildingInfo),
		UnitInfo:                 bunmodel.NewUnitInfo(domainModel.UnitInfo),
		CreatedAt:                domainModel.CreatedAt,
		PlayerDataUpdatedAt:      domainModel.PlayerDataUpdatedAt,
		PlayerSnapshotsCreatedAt: domainModel.PlayerSnapshotsCreatedAt,
		TribeDataUpdatedAt:       domainModel.TribeDataUpdatedAt,
		TribeSnapshotsCreatedAt:  domainModel.TribeSnapshotsCreatedAt,
		VillageDataUpdatedAt:     domainModel.VillageDataUpdatedAt,
		EnnoblementDataUpdatedAt: domainModel.EnnoblementDataUpdatedAt,
		VersionCode:              domainModel.VersionCode,
	}
	assert.Equal(t, domainModel, dbModel.ToDomain())
}
