package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"github.com/uptrace/bun"
)

type Tribe struct {
	bun.BaseModel `bun:"table:tribes,alias:tribe"`

	ID             int64     `bun:"id,nullzero,pk"`
	ServerKey      string    `bun:"server_key,nullzero,pk"`
	Name           string    `bun:"name,nullzero"`
	Tag            string    `bun:"tag,nullzero"`
	NumMembers     int64     `bun:"num_members"`
	NumVillages    int64     `bun:"num_villages"`
	Points         int64     `bun:"points"`
	AllPoints      int64     `bun:"all_points"`
	Rank           int64     `bun:"rank"`
	Dominance      float64   `bun:"dominance"`
	ProfileURL     string    `bun:"profile_url,nullzero"`
	BestRank       int64     `bun:"best_rank"`
	BestRankAt     time.Time `bun:"best_rank_at,nullzero"`
	MostPoints     int64     `bun:"most_points"`
	MostPointsAt   time.Time `bun:"most_points_at,nullzero"`
	MostVillages   int64     `bun:"most_villages"`
	MostVillagesAt time.Time `bun:"most_villages_at,nullzero"`
	CreatedAt      time.Time `bun:"created_at,nullzero"`
	DeletedAt      time.Time `bun:"deleted_at,nullzero"`

	OpponentsDefeated
}

func NewTribe(p domain.CreateTribeParams) Tribe {
	return Tribe{
		OpponentsDefeated: OpponentsDefeated(p.OpponentsDefeated),
		ID:                p.ID,
		ServerKey:         p.ServerKey,
		Name:              p.Name,
		Tag:               p.Tag,
		NumVillages:       p.NumVillages,
		NumMembers:        p.NumMembers,
		Points:            p.Points,
		AllPoints:         p.AllPoints,
		Rank:              p.Rank,
		ProfileURL:        p.ProfileURL,
		BestRank:          p.BestRank,
		BestRankAt:        p.BestRankAt,
		MostPoints:        p.MostPoints,
		MostPointsAt:      p.MostPointsAt,
		MostVillages:      p.MostVillages,
		MostVillagesAt:    p.MostVillagesAt,
		CreatedAt:         time.Now(),
	}
}

func (t Tribe) ToDomain() domain.Tribe {
	return domain.Tribe{
		BaseTribe: domain.BaseTribe{
			OpponentsDefeated: domain.OpponentsDefeated(t.OpponentsDefeated),
			ID:                t.ID,
			Name:              t.Name,
			Tag:               t.Tag,
			NumMembers:        t.NumMembers,
			NumVillages:       t.NumVillages,
			Points:            t.Points,
			AllPoints:         t.AllPoints,
			Rank:              t.Rank,
			ProfileURL:        t.ProfileURL,
		},
		Dominance:      t.Dominance,
		BestRank:       t.BestRank,
		BestRankAt:     t.BestRankAt,
		MostPoints:     t.MostPoints,
		MostPointsAt:   t.MostPointsAt,
		MostVillages:   t.MostVillages,
		MostVillagesAt: t.MostVillagesAt,
		ServerKey:      t.ServerKey,
		CreatedAt:      t.CreatedAt,
		DeletedAt:      t.DeletedAt,
	}
}

func (t Tribe) ToMeta() domain.TribeMeta {
	return domain.TribeMeta{
		ID:         t.ID,
		Name:       t.Name,
		Tag:        t.Tag,
		ProfileURL: t.ProfileURL,
	}
}
