package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/uptrace/bun"
)

type TribeChange struct {
	bun.BaseModel `bun:"table:tribe_changes,alias:tc"`

	ID         int64     `bun:"id,pk,autoincrement,identity"`
	PlayerID   int64     `bun:"player_id,nullzero"`
	Player     Player    `bun:"player,rel:belongs-to,join:player_id=id,join:server_key=server_key"`
	NewTribeID int64     `bun:"new_tribe_id,nullzero"`
	NewTribe   Tribe     `bun:"new_tribe,rel:belongs-to,join:new_tribe_id=id,join:server_key=server_key"`
	OldTribeID int64     `bun:"old_tribe_id,nullzero"`
	OldTribe   Tribe     `bun:"old_tribe,rel:belongs-to,join:old_tribe_id=id,join:server_key=server_key"`
	ServerKey  string    `bun:"server_key,nullzero"`
	CreatedAt  time.Time `bun:"created_at,nullzero"`
}

func NewTribeChange(p domain.CreateTribeChangeParams) TribeChange {
	return TribeChange{
		PlayerID:   p.PlayerID,
		NewTribeID: p.NewTribeID,
		OldTribeID: p.OldTribeID,
		ServerKey:  p.ServerKey,
		CreatedAt:  time.Now(),
	}
}

func (t TribeChange) ToDomain() domain.TribeChange {
	return domain.TribeChange{
		ID:         t.ID,
		PlayerID:   t.PlayerID,
		NewTribeID: t.NewTribeID,
		OldTribeID: t.OldTribeID,
		CreatedAt:  t.CreatedAt,
		ServerKey:  t.ServerKey,
	}
}

func (t TribeChange) ToDomainWithRelations() domain.TribeChangeWithRelations {
	res := domain.TribeChangeWithRelations{
		TribeChange: t.ToDomain(),
		Player:      t.Player.ToMeta(),
	}

	if t.NewTribe.ID > 0 {
		res.NewTribe = domain.NullTribeMeta{
			Valid: true,
			Tribe: t.NewTribe.ToMeta(),
		}
	}

	if t.OldTribe.ID > 0 {
		res.OldTribe = domain.NullTribeMeta{
			Valid: true,
			Tribe: t.OldTribe.ToMeta(),
		}
	}

	return res
}
