package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestTribeChange(t *testing.T) {
	t.Parallel()

	var id int64 = 12342
	params := domain.CreateTribeChangeParams{
		PlayerID:   1111,
		OldTribeID: 1112,
		NewTribeID: 1113,
		ServerKey:  "pl151",
	}

	result := bunmodel.NewTribeChange(params)
	result.ID = id

	tc := result.ToDomain()
	assert.Equal(t, id, tc.ID)
	assert.Equal(t, params.PlayerID, tc.PlayerID)
	assert.Equal(t, params.OldTribeID, tc.OldTribeID)
	assert.Equal(t, params.NewTribeID, tc.NewTribeID)
	assert.Equal(t, params.ServerKey, tc.ServerKey)
	assert.WithinDuration(t, time.Now(), tc.CreatedAt, time.Second)

	result.Player = bunmodel.Player{
		ID: 11231,
	}
	result.NewTribe = bunmodel.Tribe{
		ID: 11232,
	}
	result.OldTribe = bunmodel.Tribe{
		ID: 11234,
	}

	tcWithRelations := result.ToDomainWithRelations()
	assert.Equal(t, tc, tcWithRelations.TribeChange)
	assert.Equal(t, result.Player.ID, tcWithRelations.Player.ID)
	assert.True(t, tcWithRelations.NewTribe.Valid)
	assert.Equal(t, result.NewTribe.ID, tcWithRelations.NewTribe.Tribe.ID)
	assert.True(t, tcWithRelations.OldTribe.Valid)
	assert.Equal(t, result.OldTribe.ID, tcWithRelations.OldTribe.Tribe.ID)
}
