package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/uptrace/bun"
)

type TribeSnapshot struct {
	bun.BaseModel `bun:"table:tribe_snapshots,alias:ts"`

	ID          int64     `bun:"id,pk,autoincrement,identity"`
	TribeID     int64     `bun:"tribe_id,nullzero"`
	ServerKey   string    `bun:"server_key,nullzero"`
	NumMembers  int64     `bun:"num_members"`
	NumVillages int64     `bun:"num_villages"`
	Points      int64     `bun:"points"`
	AllPoints   int64     `bun:"all_points"`
	Rank        int64     `bun:"rank"`
	Dominance   float64   `bun:"dominance"`
	Date        time.Time `bun:"date,nullzero"`
	CreatedAt   time.Time `bun:"created_at,nullzero"`

	OpponentsDefeated
}

func NewTribeSnapshot(p domain.CreateTribeSnapshotParams) TribeSnapshot {
	return TribeSnapshot{
		OpponentsDefeated: OpponentsDefeated(p.OpponentsDefeated),
		TribeID:           p.TribeID,
		ServerKey:         p.ServerKey,
		NumMembers:        p.NumMembers,
		NumVillages:       p.NumVillages,
		Points:            p.Points,
		AllPoints:         p.AllPoints,
		Rank:              p.Rank,
		Dominance:         p.Dominance,
		Date:              p.Date,
		CreatedAt:         time.Now(),
	}
}

func (ps TribeSnapshot) ToDomain() domain.TribeSnapshot {
	return domain.TribeSnapshot{
		OpponentsDefeated: domain.OpponentsDefeated(ps.OpponentsDefeated),
		ID:                ps.ID,
		TribeID:           ps.TribeID,
		ServerKey:         ps.ServerKey,
		NumMembers:        ps.NumMembers,
		NumVillages:       ps.NumVillages,
		Points:            ps.Points,
		AllPoints:         ps.AllPoints,
		Rank:              ps.Rank,
		Dominance:         ps.Dominance,
		Date:              ps.Date,
		CreatedAt:         ps.CreatedAt,
	}
}
