package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestTribe(t *testing.T) {
	t.Parallel()

	deletedAt := time.Now().Add(-100 * time.Hour)
	params := domain.CreateTribeParams{
		BaseTribe: domain.BaseTribe{
			OpponentsDefeated: domain.OpponentsDefeated{
				RankAtt:    1,
				ScoreAtt:   2,
				RankDef:    3,
				ScoreDef:   4,
				RankSup:    5,
				ScoreSup:   6,
				RankTotal:  7,
				ScoreTotal: 8,
			},
			ID:          999,
			Name:        "name 999",
			Tag:         "tag 999",
			NumMembers:  2,
			NumVillages: 3,
			Points:      4,
			AllPoints:   5,
			Rank:        6,
			ProfileURL:  "tribe-999",
		},
		BestRank:       111,
		BestRankAt:     time.Now().Add(-5 * time.Minute),
		MostPoints:     222,
		MostPointsAt:   time.Now().Add(-1 * time.Minute),
		MostVillages:   333,
		MostVillagesAt: time.Now().Add(-12 * time.Minute),
		ServerKey:      "pl151",
	}

	result := bunmodel.NewTribe(params)
	result.DeletedAt = deletedAt

	tribe := result.ToDomain()
	assert.EqualValues(t, params.OpponentsDefeated, tribe.OpponentsDefeated)
	assert.Equal(t, params.ID, tribe.ID)
	assert.Equal(t, params.Name, tribe.Name)
	assert.Equal(t, params.Tag, tribe.Tag)
	assert.Equal(t, params.NumMembers, tribe.NumMembers)
	assert.Equal(t, params.NumVillages, tribe.NumVillages)
	assert.Equal(t, params.Points, tribe.Points)
	assert.Equal(t, params.AllPoints, tribe.AllPoints)
	assert.Equal(t, params.Rank, tribe.Rank)
	assert.Equal(t, params.ProfileURL, tribe.ProfileURL)
	assert.Equal(t, params.BestRank, tribe.BestRank)
	assert.Equal(t, params.BestRankAt, tribe.BestRankAt)
	assert.Equal(t, params.MostPoints, tribe.MostPoints)
	assert.Equal(t, params.MostPointsAt, tribe.MostPointsAt)
	assert.Equal(t, params.MostVillages, tribe.MostVillages)
	assert.Equal(t, params.MostVillagesAt, tribe.MostVillagesAt)
	assert.Equal(t, params.ServerKey, tribe.ServerKey)
	assert.WithinDuration(t, time.Now(), tribe.CreatedAt, time.Second)
	assert.Equal(t, deletedAt, tribe.DeletedAt)

	meta := result.ToMeta()
	assert.Equal(t, params.ID, meta.ID)
	assert.Equal(t, params.Name, meta.Name)
	assert.Equal(t, params.Tag, meta.Tag)
	assert.Equal(t, params.ProfileURL, meta.ProfileURL)

}
