package bunmodel

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/uptrace/bun"
)

type Village struct {
	bun.BaseModel `bun:"table:villages,alias:village"`

	ID         int64     `bun:"id,nullzero,pk"`
	ServerKey  string    `bun:"server_key,nullzero,pk"`
	Name       string    `bun:"name,nullzero"`
	Points     int64     `bun:"points"`
	X          int64     `bun:"x"`
	Y          int64     `bun:"y"`
	Continent  string    `bun:"continent"`
	Bonus      int64     `bun:"bonus"`
	PlayerID   int64     `bun:"player_id,nullzero"`
	Player     Player    `bun:"player,rel:belongs-to,join:player_id=id,join:server_key=server_key"`
	ProfileURL string    `bun:"profile_url,nullzero"`
	CreatedAt  time.Time `bun:"created_at,nullzero"`
}

func NewVillage(p domain.CreateVillageParams) Village {
	return Village{
		ID:         p.ID,
		Name:       p.Name,
		Points:     p.Points,
		X:          p.X,
		Y:          p.Y,
		Continent:  p.Continent,
		Bonus:      p.Bonus,
		PlayerID:   p.PlayerID,
		ServerKey:  p.ServerKey,
		ProfileURL: p.ProfileURL,
		CreatedAt:  time.Now(),
	}
}

func (v Village) ToDomain() domain.Village {
	return domain.Village{
		BaseVillage: domain.BaseVillage{
			ID:         v.ID,
			Name:       v.Name,
			Points:     v.Points,
			X:          v.X,
			Y:          v.Y,
			Continent:  v.Continent,
			Bonus:      v.Bonus,
			PlayerID:   v.PlayerID,
			ProfileURL: v.ProfileURL,
		},
		ServerKey: v.ServerKey,
		CreatedAt: v.CreatedAt,
	}
}

func (v Village) ToDomainWithRelations() domain.VillageWithRelations {
	res := domain.VillageWithRelations{
		Village: v.ToDomain(),
	}

	if v.Player.ID > 0 {
		res.Player = domain.NullPlayerMetaWithRelations{
			Valid:  true,
			Player: v.Player.ToMetaWithRelations(),
		}
	}

	return res
}

func (v Village) ToMeta() domain.VillageMeta {
	return domain.VillageMeta{
		ID:         v.ID,
		Name:       v.Name,
		X:          v.X,
		Y:          v.Y,
		Continent:  v.Continent,
		ProfileURL: v.ProfileURL,
	}
}
