package bunmodel_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/bunmodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestVillage(t *testing.T) {
	t.Parallel()

	params := domain.CreateVillageParams{
		BaseVillage: domain.BaseVillage{
			ID:         999,
			Name:       "name 999",
			Points:     4,
			X:          500,
			Y:          500,
			Continent:  "K55",
			Bonus:      0,
			PlayerID:   1235,
			ProfileURL: "profile-999",
		},
		ServerKey: "pl151",
	}

	result := bunmodel.NewVillage(params)

	village := result.ToDomain()
	assert.Equal(t, params.ID, village.ID)
	assert.Equal(t, params.Name, village.Name)
	assert.Equal(t, params.Points, village.Points)
	assert.Equal(t, params.X, village.X)
	assert.Equal(t, params.Continent, village.Continent)
	assert.Equal(t, params.Bonus, village.Bonus)
	assert.Equal(t, params.PlayerID, village.PlayerID)
	assert.Equal(t, params.ProfileURL, village.ProfileURL)
	assert.Equal(t, params.ServerKey, village.ServerKey)
	assert.WithinDuration(t, time.Now(), village.CreatedAt, time.Second)

	meta := result.ToMeta()
	assert.Equal(t, params.ID, meta.ID)
	assert.Equal(t, params.Name, meta.Name)
	assert.Equal(t, params.X, meta.X)
	assert.Equal(t, params.Y, meta.Y)
	assert.Equal(t, params.Continent, meta.Continent)
	assert.Equal(t, params.ProfileURL, meta.ProfileURL)

	result.Player = bunmodel.Player{
		ID: 12321,
		Tribe: bunmodel.Tribe{
			ID: 1234112,
		},
	}

	villageWithRelations := result.ToDomainWithRelations()
	assert.Equal(t, village, villageWithRelations.Village)
	assert.WithinDuration(t, time.Now(), villageWithRelations.CreatedAt, time.Second)
	assert.True(t, villageWithRelations.Player.Valid)
	assert.Equal(t, result.Player.ID, villageWithRelations.Player.Player.ID)
	assert.True(t, villageWithRelations.Player.Player.Tribe.Valid)
	assert.Equal(t, result.Player.Tribe.ID, villageWithRelations.Player.Player.Tribe.Tribe.ID)
}
