package adapter

import (
	"context"
	"fmt"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/publishermodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"github.com/ThreeDotsLabs/watermill/message"
)

type ServerWatermillPublisher struct {
	publisher message.Publisher
	marshaler WatermillMarshaler
}

func NewServerWatermillPublisher(publisher message.Publisher, marshaler WatermillMarshaler) *ServerWatermillPublisher {
	return &ServerWatermillPublisher{publisher: publisher, marshaler: marshaler}
}

func (s *ServerWatermillPublisher) CmdRefresh(ctx context.Context, payloads ...domain.RefreshServersCmdPayload) error {
	msgs := make([]*message.Message, 0, len(payloads))
	for _, p := range payloads {
		msg, err := s.marshaler.Marshal(publishermodel.RefreshServersCmdPayload(p))
		if err != nil {
			return fmt.Errorf("%s: couldn't marshal RefreshServersCmdPayload: %w", p.VersionCode, err)
		}

		msg.SetContext(ctx)

		msgs = append(msgs, msg)
	}

	topic := "servers.cmd.refresh"
	if err := s.publisher.Publish(topic, msgs...); err != nil {
		return fmt.Errorf("couldn't publish message with topic %s: %w", topic, err)
	}

	return nil
}
