package adapter_test

import (
	"context"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter"
	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/publishermodel"
	"github.com/ThreeDotsLabs/watermill"

	"github.com/stretchr/testify/assert"

	"github.com/ThreeDotsLabs/watermill/message/subscriber"

	"github.com/stretchr/testify/require"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestServerWatermillPublisher_CmdRefresh(t *testing.T) {
	t.Parallel()

	marshaler := adapter.JSONWatermillMarshaler{
		NewUUID: watermill.NewUUID,
	}
	pubSub := newPubSub(t)

	payloads := []domain.RefreshServersCmdPayload{
		{
			URL:         "https://host1.com",
			VersionCode: "vc1",
		},
		{
			URL:         "https://host2.com",
			VersionCode: "vc2",
		},
	}

	msgs, err := pubSub.Subscribe(context.Background(), "servers.cmd.refresh")
	require.NoError(t, err)

	require.NoError(t, adapter.NewServerWatermillPublisher(pubSub, marshaler).CmdRefresh(context.Background(), payloads...))

	receivedMsgs, _ := subscriber.BulkRead(msgs, len(payloads), time.Second)
	require.Len(t, receivedMsgs, len(payloads))
	for _, m := range receivedMsgs {
		var received publishermodel.RefreshServersCmdPayload
		assert.NoError(t, marshaler.Unmarshal(m, &received))
		assert.Contains(t, payloads, domain.RefreshServersCmdPayload{
			URL:         received.URL,
			VersionCode: received.VersionCode,
		})
	}
}
