package adapter

import (
	"context"
	"fmt"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/publishermodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/ThreeDotsLabs/watermill/message"
)

type SnapshotWatermillPublisher struct {
	publisher message.Publisher
	marshaler WatermillMarshaler
}

func NewSnapshotWatermillPublisher(publisher message.Publisher, marshaler WatermillMarshaler) *SnapshotWatermillPublisher {
	return &SnapshotWatermillPublisher{publisher: publisher, marshaler: marshaler}
}

func (s *SnapshotWatermillPublisher) CmdCreatePlayers(ctx context.Context, payloads ...domain.CreateSnapshotsCmdPayload) error {
	return s.publishCmdCreate(ctx, "players.cmd.create_snapshots", payloads...)
}

func (s *SnapshotWatermillPublisher) CmdCreateTribes(ctx context.Context, payloads ...domain.CreateSnapshotsCmdPayload) error {
	return s.publishCmdCreate(ctx, "tribes.cmd.create_snapshots", payloads...)
}

func (s *SnapshotWatermillPublisher) publishCmdCreate(
	ctx context.Context,
	topic string,
	payloads ...domain.CreateSnapshotsCmdPayload,
) error {
	msgs := make([]*message.Message, 0, len(payloads))
	for _, p := range payloads {
		msg, err := s.marshaler.Marshal(publishermodel.CreateSnapshotsCmdPayload(p))
		if err != nil {
			return fmt.Errorf("%s: couldn't marshal CreateSnapshotsCmdPayload: %w", p.Key, err)
		}

		msg.SetContext(ctx)

		msgs = append(msgs, msg)
	}

	if err := s.publisher.Publish(topic, msgs...); err != nil {
		return fmt.Errorf("couldn't publish message with topic %s: %w", topic, err)
	}

	return nil
}
