package adapter_test

import (
	"context"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter"
	"gitea.dwysokinski.me/twhelp/core/internal/adapter/internal/publishermodel"
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/ThreeDotsLabs/watermill"
	"github.com/ThreeDotsLabs/watermill/message/subscriber"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestSnapshotWatermillPublisher_CmdCreate(t *testing.T) {
	t.Parallel()

	marshaler := adapter.JSONWatermillMarshaler{
		NewUUID: watermill.NewUUID,
	}
	pubSub := newPubSub(t)
	publisher := adapter.NewSnapshotWatermillPublisher(pubSub, marshaler)

	payloads := []domain.CreateSnapshotsCmdPayload{
		{
			Key:         "pl151",
			VersionCode: "pl",
			Date:        time.Now(),
		},
		{
			Key:         "pl182",
			VersionCode: "pl",
			Date:        time.Now().Add(-25 * time.Hour),
		},
	}

	tests := []struct {
		topic string
		f     func(ctx context.Context, payloads ...domain.CreateSnapshotsCmdPayload) error
	}{
		{
			topic: "players.cmd.create_snapshots",
			f:     publisher.CmdCreatePlayers,
		},
		{
			topic: "tribes.cmd.create_snapshots",
			f:     publisher.CmdCreateTribes,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.topic, func(t *testing.T) {
			t.Parallel()

			msgs, err := pubSub.Subscribe(context.Background(), tt.topic)
			require.NoError(t, err)

			require.NoError(t, tt.f(context.Background(), payloads...))

			receivedMsgs, _ := subscriber.BulkRead(msgs, len(payloads), time.Second)
			require.Len(t, receivedMsgs, len(payloads))
			for _, m := range receivedMsgs {
				var received publishermodel.CreateSnapshotsCmdPayload
				assert.NoError(t, marshaler.Unmarshal(m, &received))

				found := false
				for _, payload := range payloads {
					if payload.VersionCode == received.VersionCode && payload.Key == received.Key {
						found = true
						break
					}
				}
				assert.True(t, found)
			}
		})
	}
}
