package adapter_test

import (
	"context"
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"github.com/stretchr/testify/assert"
)

func TestVersionBunRepository_List_ListCount(t *testing.T) {
	t.Parallel()

	if testing.Short() {
		t.Skip("skipping long-running test")
	}

	repo := adapter.NewVersionBunRepository(newBunDB(t))

	allVersions := []string{
		"pl",
		"en",
		"de",
		"uk",
		"it",
		"fr",
		"us",
		"nl",
		"es",
		"ro",
		"ru",
		"gr",
		"tr",
		"cs",
		"ch",
		"pt",
		"br",
		"hu",
		"sk",
	}

	resListCount, count, err := repo.ListCount(context.Background())
	assert.NoError(t, err)
	assert.Equal(t, int64(len(allVersions)), count)
	assert.Len(t, resListCount, len(allVersions))
	for _, versionCode := range allVersions {
		found := false
		for _, version := range resListCount {
			if version.Code == versionCode {
				found = true
				break
			}
		}
		assert.True(t, found, "version (code=%s) not found", versionCode)
	}

	resList, err := repo.List(context.Background())
	assert.NoError(t, err)
	assert.Equal(t, resListCount, resList)
}

func TestVersionBunRepository_GetByCode(t *testing.T) {
	t.Parallel()

	if testing.Short() {
		t.Skip("skipping long-running test")
	}

	repo := adapter.NewVersionBunRepository(newBunDB(t))

	t.Run("OK", func(t *testing.T) {
		t.Parallel()

		version, err := repo.GetByCode(context.Background(), "pl")
		assert.NoError(t, err)
		assert.Equal(t, "pl", version.Code)
	})

	t.Run("ERR: version not found", func(t *testing.T) {
		t.Parallel()

		code := "pl2"
		version, err := repo.GetByCode(context.Background(), code)
		assert.ErrorIs(t, err, domain.VersionNotFoundError{VerCode: code})
		assert.Zero(t, version)
	})
}
