package adapter

import (
	"encoding/json"

	"github.com/ThreeDotsLabs/watermill/message"
)

type WatermillMarshaler interface {
	Marshal(v any) (*message.Message, error)
	Unmarshal(msg *message.Message, v any) error
}

type JSONWatermillMarshaler struct {
	NewUUID func() string
}

func (m JSONWatermillMarshaler) Marshal(v any) (*message.Message, error) {
	b, err := json.Marshal(v)
	if err != nil {
		return nil, err
	}

	msg := message.NewMessage(m.NewUUID(), b)
	msg.Metadata.Set("Content-Type", "application/json")

	return msg, nil
}

func (m JSONWatermillMarshaler) Unmarshal(msg *message.Message, v any) error {
	return json.Unmarshal(msg.Payload, v)
}
