package adapter_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/adapter"
	"github.com/ThreeDotsLabs/watermill"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

type testPayload struct {
	ID   string    `json:"id"`
	Num  int       `json:"num"`
	When time.Time `json:"when"`
}

func TestJSONWatermillMarshaler(t *testing.T) {
	t.Parallel()

	payloadToMarshal := testPayload{
		ID:   watermill.NewUUID(),
		Num:  123,
		When: time.Date(2022, time.July, 30, 14, 13, 12, 0, time.UTC),
	}
	marshaler := adapter.JSONWatermillMarshaler{NewUUID: watermill.NewUUID}

	m, err := marshaler.Marshal(payloadToMarshal)
	require.NoError(t, err)

	payloadToUnmarshal := testPayload{}
	require.NoError(t, marshaler.Unmarshal(m, &payloadToUnmarshal))

	assert.Equal(t, payloadToMarshal, payloadToUnmarshal)
}
