package domain

import (
	"errors"
	"strconv"
	"strings"
)

const coordsSubstrings = 2

var ErrIncorrectCoordsFormat = errors.New("incorrect coords format, should be x|y")

type Coords struct {
	X int64
	Y int64
}

func NewCoords(s string) (Coords, error) {
	coords := strings.SplitN(s, "|", coordsSubstrings)
	if len(coords) != coordsSubstrings {
		return Coords{}, ErrIncorrectCoordsFormat
	}

	x, err := strconv.ParseInt(coords[0], 10, 64)
	if err != nil || x <= 0 {
		return Coords{}, ErrIncorrectCoordsFormat
	}

	y, err := strconv.ParseInt(coords[1], 10, 64)
	if err != nil || y <= 0 {
		return Coords{}, ErrIncorrectCoordsFormat
	}

	return Coords{
		X: x,
		Y: y,
	}, nil
}
