package domain_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestNewCoords(t *testing.T) {
	t.Parallel()

	tests := []struct {
		coords      string
		expected    domain.Coords
		expectedErr error
	}{
		{
			coords: "571|123",
			expected: domain.Coords{
				X: 571,
				Y: 123,
			},
		},
		{
			coords:      "-571|123",
			expectedErr: domain.ErrIncorrectCoordsFormat,
		},
		{
			coords:      "571|-123",
			expectedErr: domain.ErrIncorrectCoordsFormat,
		},
		{
			coords:      " 571|123",
			expectedErr: domain.ErrIncorrectCoordsFormat,
		},
		{
			coords:      "571123",
			expectedErr: domain.ErrIncorrectCoordsFormat,
		},
		{
			coords:      "571|123|",
			expectedErr: domain.ErrIncorrectCoordsFormat,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.coords, func(t *testing.T) {
			t.Parallel()

			coords, err := domain.NewCoords(tt.coords)
			assert.ErrorIs(t, err, tt.expectedErr)
			assert.Equal(t, tt.expected, coords)
		})
	}
}
