package domain

import (
	"fmt"
	"time"
)

type BaseEnnoblement struct {
	VillageID  int64
	NewOwnerID int64
	NewTribeID int64
	OldOwnerID int64
	OldTribeID int64
	Points     int64
	CreatedAt  time.Time
}

type Ennoblement struct {
	BaseEnnoblement

	ID        int64
	ServerKey string
}

type EnnoblementWithRelations struct {
	Ennoblement

	Village  VillageMeta
	NewOwner NullPlayerMeta
	NewTribe NullTribeMeta
	OldOwner NullPlayerMeta
	OldTribe NullTribeMeta
}

type CreateEnnoblementParams struct {
	BaseEnnoblement

	ServerKey string
}

type EnnoblementSortBy uint8

const (
	EnnoblementSortByCreatedAt EnnoblementSortBy = iota
)

func newEnnoblementSortBy(s string) (EnnoblementSortBy, error) {
	switch s {
	case "createdAt":
		return EnnoblementSortByCreatedAt, nil
	default:
		return 0, fmt.Errorf("%w: \"%s\"", ErrUnsupportedSortBy, s)
	}
}

type EnnoblementSort struct {
	By        EnnoblementSortBy
	Direction SortDirection
}

func NewEnnoblementSort(by, dir string) (EnnoblementSort, error) {
	convBy, err := newEnnoblementSortBy(by)
	if err != nil {
		return EnnoblementSort{}, err
	}

	convDir, err := newSortDirection(dir)
	if err != nil {
		return EnnoblementSort{}, err
	}

	return EnnoblementSort{
		By:        convBy,
		Direction: convDir,
	}, nil
}

type ListEnnoblementsParams struct {
	ServerKeys   []string
	CreatedAtGTE NullTime
	CreatedAtLTE NullTime
	// PlayerID filters out ennoblements whose NewOwnerID OR OldOwnerID isn't equal to the provided value.
	PlayerID NullInt64
	// TribeID filters out ennoblements whose NewTribeID OR OldTribeID isn't equal to the provided value.
	TribeID    NullInt64
	VillageID  NullInt64
	Pagination Pagination
	Sort       []EnnoblementSort
}

type RefreshEnnoblementsCmdPayload struct {
	Key         string
	URL         string
	VersionCode string
}
