package domain

import (
	"fmt"
	"time"
)

type OpenServer struct {
	Key string
	URL string
}

type Server struct {
	Key                      string
	URL                      string
	Open                     bool
	Special                  bool
	NumPlayers               int64
	NumTribes                int64
	NumVillages              int64
	NumPlayerVillages        int64
	NumBarbarianVillages     int64
	NumBonusVillages         int64
	Config                   ServerConfig
	BuildingInfo             BuildingInfo
	UnitInfo                 UnitInfo
	CreatedAt                time.Time
	PlayerDataUpdatedAt      time.Time
	PlayerSnapshotsCreatedAt time.Time
	TribeDataUpdatedAt       time.Time
	TribeSnapshotsCreatedAt  time.Time
	VillageDataUpdatedAt     time.Time
	EnnoblementDataUpdatedAt time.Time
	VersionCode              string
}

type ServerMeta struct {
	Key  string
	URL  string
	Open bool
}

type CreateServerParams struct {
	Key         string
	URL         string
	Open        bool
	VersionCode string
}

type UpdateServerParams struct {
	Config                   NullServerConfig
	BuildingInfo             NullBuildingInfo
	UnitInfo                 NullUnitInfo
	NumPlayers               NullInt64
	PlayerDataUpdatedAt      NullTime
	PlayerSnapshotsCreatedAt NullTime
	NumTribes                NullInt64
	TribeDataUpdatedAt       NullTime
	TribeSnapshotsCreatedAt  NullTime
	NumVillages              NullInt64
	NumPlayerVillages        NullInt64
	NumBarbarianVillages     NullInt64
	NumBonusVillages         NullInt64
	VillageDataUpdatedAt     NullTime
	EnnoblementDataUpdatedAt NullTime
}

//nolint:gocyclo
func (u UpdateServerParams) IsZero() bool {
	return !u.Config.Valid &&
		!u.BuildingInfo.Valid &&
		!u.UnitInfo.Valid &&
		!u.NumPlayers.Valid &&
		!u.PlayerDataUpdatedAt.Valid &&
		!u.PlayerSnapshotsCreatedAt.Valid &&
		!u.NumTribes.Valid &&
		!u.TribeDataUpdatedAt.Valid &&
		!u.TribeSnapshotsCreatedAt.Valid &&
		!u.NumVillages.Valid &&
		!u.NumPlayerVillages.Valid &&
		!u.NumBarbarianVillages.Valid &&
		!u.NumBonusVillages.Valid &&
		!u.VillageDataUpdatedAt.Valid &&
		!u.EnnoblementDataUpdatedAt.Valid
}

type ListServersParams struct {
	Keys                       []string
	Special                    NullBool
	Open                       NullBool
	PlayerSnapshotsCreatedAtLT time.Time
	TribeSnapshotsCreatedAtLT  time.Time
	VersionCodes               []string
	Pagination                 Pagination
}

type RefreshServersCmdPayload struct {
	URL         string
	VersionCode string
}

type ServerNotFoundError struct {
	Key string
}

func (e ServerNotFoundError) Error() string {
	return fmt.Sprintf("server (key=%s) not found", e.Key)
}

func (e ServerNotFoundError) UserError() string {
	return e.Error()
}

func (e ServerNotFoundError) Code() ErrorCode {
	return ErrorCodeEntityNotFound
}
