package domain_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestNewPlayerSnapshotSort(t *testing.T) {
	t.Parallel()

	tests := []struct {
		by          string
		dir         string
		output      domain.PlayerSnapshotSort
		expectedErr error
	}{
		{
			by:  "id",
			dir: "desc",
			output: domain.PlayerSnapshotSort{
				By:        domain.PlayerSnapshotSortByID,
				Direction: domain.SortDirectionDESC,
			},
		},
		{
			by:  "date",
			dir: "asc",
			output: domain.PlayerSnapshotSort{
				By:        domain.PlayerSnapshotSortByDate,
				Direction: domain.SortDirectionASC,
			},
		},
		{
			by:          "unsupported",
			dir:         "asc",
			expectedErr: domain.ErrUnsupportedSortBy,
		},
		{
			by:          "id",
			dir:         "unsupported",
			expectedErr: domain.ErrUnsupportedSortDirection,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.by+":"+tt.dir, func(t *testing.T) {
			t.Parallel()

			res, err := domain.NewPlayerSnapshotSort(tt.by, tt.dir)
			assert.ErrorIs(t, err, tt.expectedErr)
			assert.Equal(t, tt.output, res)
		})
	}
}

func TestNewTribeSnapshotSort(t *testing.T) {
	t.Parallel()

	tests := []struct {
		by          string
		dir         string
		output      domain.TribeSnapshotSort
		expectedErr error
	}{
		{
			by:  "id",
			dir: "desc",
			output: domain.TribeSnapshotSort{
				By:        domain.TribeSnapshotSortByID,
				Direction: domain.SortDirectionDESC,
			},
		},
		{
			by:  "date",
			dir: "asc",
			output: domain.TribeSnapshotSort{
				By:        domain.TribeSnapshotSortByDate,
				Direction: domain.SortDirectionASC,
			},
		},
		{
			by:          "unsupported",
			dir:         "asc",
			expectedErr: domain.ErrUnsupportedSortBy,
		},
		{
			by:          "id",
			dir:         "unsupported",
			expectedErr: domain.ErrUnsupportedSortDirection,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.by+":"+tt.dir, func(t *testing.T) {
			t.Parallel()

			res, err := domain.NewTribeSnapshotSort(tt.by, tt.dir)
			assert.ErrorIs(t, err, tt.expectedErr)
			assert.Equal(t, tt.output, res)
		})
	}
}
