package domain_test

import (
	"testing"

	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestNewTribeSort(t *testing.T) {
	t.Parallel()

	tests := []struct {
		by          string
		dir         string
		output      domain.TribeSort
		expectedErr error
	}{
		{
			by:  "id",
			dir: "asc",
			output: domain.TribeSort{
				By:        domain.TribeSortByID,
				Direction: domain.SortDirectionASC,
			},
		},
		{
			by:  "scoreAtt",
			dir: "desc",
			output: domain.TribeSort{
				By:        domain.TribeSortByScoreAtt,
				Direction: domain.SortDirectionDESC,
			},
		},
		{
			by:  "scoreDef",
			dir: "desc",
			output: domain.TribeSort{
				By:        domain.TribeSortByScoreDef,
				Direction: domain.SortDirectionDESC,
			},
		},
		{
			by:  "scoreTotal",
			dir: "asc",
			output: domain.TribeSort{
				By:        domain.TribeSortByScoreTotal,
				Direction: domain.SortDirectionASC,
			},
		},
		{
			by:  "points",
			dir: "desc",
			output: domain.TribeSort{
				By:        domain.TribeSortByPoints,
				Direction: domain.SortDirectionDESC,
			},
		},
		{
			by:  "dominance",
			dir: "asc",
			output: domain.TribeSort{
				By:        domain.TribeSortByDominance,
				Direction: domain.SortDirectionASC,
			},
		},
		{
			by:  "deletedAt",
			dir: "asc",
			output: domain.TribeSort{
				By:        domain.TribeSortByDeletedAt,
				Direction: domain.SortDirectionASC,
			},
		},
		{
			by:          "unsupported",
			dir:         "asc",
			expectedErr: domain.ErrUnsupportedSortBy,
		},
		{
			by:          "dominance",
			dir:         "unsupported",
			expectedErr: domain.ErrUnsupportedSortDirection,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.by+":"+tt.dir, func(t *testing.T) {
			t.Parallel()

			res, err := domain.NewTribeSort(tt.by, tt.dir)
			assert.ErrorIs(t, err, tt.expectedErr)
			assert.Equal(t, tt.output, res)
		})
	}
}

func TestTribeNotFoundError(t *testing.T) {
	t.Parallel()

	err := domain.TribeNotFoundError{
		ID: 1234,
	}
	var _ domain.Error = err
	assert.Equal(t, "tribe (id=1234) not found", err.Error())
	assert.Equal(t, err.Error(), err.UserError())
	assert.Equal(t, domain.ErrorCodeEntityNotFound, err.Code())
}
