package domain_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"github.com/stretchr/testify/assert"
)

func TestBaseVillage_FullName(t *testing.T) {
	t.Parallel()

	v := domain.BaseVillage{
		ID:        1234,
		Name:      "Village",
		X:         450,
		Y:         450,
		Continent: "K44",
	}
	assert.Equal(t, "Village (450|450) K44", v.FullName())
}

func TestBaseVillage_Coords(t *testing.T) {
	t.Parallel()

	v := domain.BaseVillage{
		X: 450,
		Y: 450,
	}
	assert.Equal(t, "450|450", v.Coords())
}

func TestVillageMeta_FullName(t *testing.T) {
	t.Parallel()

	v := domain.BaseVillage{
		ID:        1234,
		Name:      "Village",
		X:         450,
		Y:         450,
		Continent: "K44",
	}
	meta := domain.VillageMeta{
		ID:        v.ID,
		Name:      v.Name,
		X:         v.X,
		Y:         v.Y,
		Continent: v.Continent,
	}
	assert.Equal(t, v.FullName(), meta.FullName())
}

func TestVillageNotFoundError(t *testing.T) {
	t.Parallel()

	err := domain.VillageNotFoundError{
		ID: 1234,
	}
	var _ domain.Error = err
	assert.Equal(t, "village (id=1234) not found", err.Error())
	assert.Equal(t, err.Error(), err.UserError())
	assert.Equal(t, domain.ErrorCodeEntityNotFound, err.Code())
}
