package msg_test

import (
	"context"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/msg/internal/model"
	"github.com/ThreeDotsLabs/watermill"
	"github.com/ThreeDotsLabs/watermill/message/subscriber"
	"github.com/google/uuid"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"gitea.dwysokinski.me/twhelp/core/internal/msg"
	"gitea.dwysokinski.me/twhelp/core/internal/msg/internal/mock"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestEnnoblementConsumer_refresh(t *testing.T) {
	t.Parallel()

	marshaler := msg.JSONMarshaler{NewUUID: uuid.NewString}
	pubSub := newPubSub(t)

	ennoblementSvc := &mock.FakeEnnoblementService{}
	ennoblementSvc.RefreshReturns(nil)

	runRouter(t, msg.NewEnnoblementConsumer(marshaler, pubSub, pubSub, watermill.NopLogger{}, ennoblementSvc))

	msgs, err := pubSub.Subscribe(context.Background(), "ennoblements.event.refreshed")
	require.NoError(t, err)

	payload := domain.RefreshEnnoblementsCmdPayload{
		Key:         "pl151",
		URL:         "https://pl151.plemiona.pl",
		VersionCode: "pl",
	}
	cmd, err := marshaler.Marshal(payload)
	require.NoError(t, err)
	require.NoError(t, pubSub.Publish("ennoblements.cmd.refresh", cmd))

	receivedMsgs, _ := subscriber.BulkRead(msgs, 1, 10*time.Second)
	require.Len(t, receivedMsgs, 1)
	var ennoblementsRefreshedPayload model.EnnoblementsRefreshedEvPayload
	assert.NoError(t, marshaler.Unmarshal(receivedMsgs[0], &ennoblementsRefreshedPayload))
	assert.Equal(t, payload.Key, ennoblementsRefreshedPayload.Key)
	assert.Equal(t, payload.URL, ennoblementsRefreshedPayload.URL)
	assert.Equal(t, payload.VersionCode, ennoblementsRefreshedPayload.VersionCode)
}
