package model

import "gitea.dwysokinski.me/twhelp/core/internal/domain"

type Building struct {
	MaxLevel        int8    `json:"maxLevel"`
	MinLevel        int8    `json:"minLevel"`
	Wood            int32   `json:"wood"`
	Stone           int32   `json:"stone"`
	Iron            int32   `json:"iron"`
	Pop             int16   `json:"pop"`
	WoodFactor      float64 `json:"woodFactor"`
	StoneFactor     float64 `json:"stoneFactor"`
	IronFactor      float64 `json:"ironFactor"`
	PopFactor       float64 `json:"popFactor"`
	BuildTime       float64 `json:"buildTime"`
	BuildTimeFactor float64 `json:"buildTimeFactor"`
} // @name Building

type BuildingInfo struct {
	Main       Building `json:"main"`
	Barracks   Building `json:"barracks"`
	Stable     Building `json:"stable"`
	Garage     Building `json:"garage"`
	Watchtower Building `json:"watchtower"`
	Snob       Building `json:"snob"`
	Smith      Building `json:"smith"`
	Place      Building `json:"place"`
	Statue     Building `json:"statue"`
	Market     Building `json:"market"`
	Wood       Building `json:"wood"`
	Stone      Building `json:"stone"`
	Iron       Building `json:"iron"`
	Farm       Building `json:"farm"`
	Storage    Building `json:"storage"`
	Hide       Building `json:"hide"`
	Wall       Building `json:"wall"`
} // @name BuildingInfo

func NewBuildingInfo(info domain.BuildingInfo) BuildingInfo {
	return BuildingInfo{
		Main:       Building(info.Main),
		Barracks:   Building(info.Barracks),
		Stable:     Building(info.Stable),
		Garage:     Building(info.Garage),
		Watchtower: Building(info.Watchtower),
		Snob:       Building(info.Snob),
		Smith:      Building(info.Smith),
		Place:      Building(info.Place),
		Statue:     Building(info.Statue),
		Market:     Building(info.Market),
		Wood:       Building(info.Wood),
		Stone:      Building(info.Stone),
		Iron:       Building(info.Iron),
		Farm:       Building(info.Farm),
		Storage:    Building(info.Storage),
		Hide:       Building(info.Hide),
		Wall:       Building(info.Wall),
	}
}

type GetBuildingInfoResp struct {
	Data BuildingInfo `json:"data"`
} // @name GetBuildingInfoResp

func NewGetBuildingInfoResp(cfg domain.BuildingInfo) GetBuildingInfoResp {
	return GetBuildingInfoResp{
		Data: NewBuildingInfo(cfg),
	}
}
