package model_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestNewBuildingInfo_NewGetBuildingInfoResp(t *testing.T) {
	t.Parallel()

	info := domain.BuildingInfo{
		Main: domain.Building{
			MaxLevel: 124,
			Wood:     12224,
		},
		Barracks: domain.Building{
			MinLevel: 22,
			Stone:    12341,
		},
		Stable: domain.Building{
			Wood: 22241,
			Pop:  111,
		},
		Garage: domain.Building{
			Stone:      11121,
			WoodFactor: 0.99,
		},
		Watchtower: domain.Building{
			Iron:        123412,
			StoneFactor: 0.878,
		},
		Snob: domain.Building{
			Pop:       789,
			PopFactor: 0.11,
		},
		Smith: domain.Building{
			BuildTime:       23.49999,
			BuildTimeFactor: 0.999,
		},
		Place: domain.Building{
			MaxLevel: 112,
		},
		Statue: domain.Building{
			MinLevel: 123,
		},
		Market: domain.Building{
			Pop: 1234,
		},
		Wood: domain.Building{
			WoodFactor: 0.998,
		},
		Stone: domain.Building{
			StoneFactor: 0.777,
		},
		Iron: domain.Building{
			IronFactor: 0.5689,
		},
		Farm: domain.Building{
			PopFactor: 0.9,
		},
		Storage: domain.Building{
			MaxLevel: 31,
		},
		Hide: domain.Building{
			MinLevel: 1,
		},
		Wall: domain.Building{
			MaxLevel:        20,
			MinLevel:        0,
			Wood:            1234,
			Stone:           1231,
			Iron:            3214,
			Pop:             1,
			WoodFactor:      0.999,
			StoneFactor:     0.998,
			IronFactor:      0.977,
			PopFactor:       2,
			BuildTime:       30.5,
			BuildTimeFactor: 0.5,
		},
	}
	assertBuildingInfo(t, info, model.NewBuildingInfo(info))
	assertBuildingInfo(t, info, model.NewGetBuildingInfoResp(info).Data)
}

func assertBuildingInfo(tb testing.TB, dbi domain.BuildingInfo, rbi model.BuildingInfo) {
	tb.Helper()

	assert.EqualValues(tb, dbi.Main, rbi.Main)
	assert.EqualValues(tb, dbi.Barracks, rbi.Barracks)
	assert.EqualValues(tb, dbi.Stable, rbi.Stable)
	assert.EqualValues(tb, dbi.Garage, rbi.Garage)
	assert.EqualValues(tb, dbi.Watchtower, rbi.Watchtower)
	assert.EqualValues(tb, dbi.Snob, rbi.Snob)
	assert.EqualValues(tb, dbi.Smith, rbi.Smith)
	assert.EqualValues(tb, dbi.Place, rbi.Place)
	assert.EqualValues(tb, dbi.Statue, rbi.Statue)
	assert.EqualValues(tb, dbi.Market, rbi.Market)
	assert.EqualValues(tb, dbi.Wood, rbi.Wood)
	assert.EqualValues(tb, dbi.Stone, rbi.Stone)
	assert.EqualValues(tb, dbi.Iron, rbi.Iron)
	assert.EqualValues(tb, dbi.Farm, rbi.Farm)
	assert.EqualValues(tb, dbi.Storage, rbi.Storage)
	assert.EqualValues(tb, dbi.Hide, rbi.Hide)
	assert.EqualValues(tb, dbi.Wall, rbi.Wall)
}
