package model_test

import (
	"encoding/json"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"
)

func TestDate_MarshalJSON(t *testing.T) {
	t.Parallel()

	tests := []struct {
		name         string
		d            model.Date
		expectedJSON string
	}{
		{
			name: "OK: zero",
			d: model.Date{
				Time: time.Time{},
			},
			expectedJSON: `"0001-01-01"`,
		},
		{
			name: "OK: 2022-07-30",
			d: model.Date{
				Time: time.Date(2022, time.July, 30, 14, 13, 12, 500, time.UTC),
			},
			expectedJSON: `"2022-07-30"`,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.name, func(t *testing.T) {
			t.Parallel()

			b, err := json.Marshal(tt.d)
			assert.NoError(t, err)
			assert.JSONEq(t, tt.expectedJSON, string(b))
		})
	}
}

func TestDate_UnmarshalJSON(t *testing.T) {
	t.Parallel()

	tests := []struct {
		name                    string
		json                    string
		expectedT               model.Date
		expectedJSONSyntaxError bool
	}{
		{
			name: "OK: 2022-07-30",
			json: `"2022-07-30"`,
			expectedT: model.Date{
				Time: time.Date(2022, time.July, 30, 0, 0, 0, 0, time.UTC),
			},
		},
		{
			name:                    "ERR: invalid time 1",
			json:                    "2022-07-30T14:13:12.0000005Z",
			expectedT:               model.Date{},
			expectedJSONSyntaxError: true,
		},
		{
			name:                    "ERR: invalid time 2",
			json:                    "hello world",
			expectedT:               model.Date{},
			expectedJSONSyntaxError: true,
		},
	}

	for _, tt := range tests {
		tt := tt

		t.Run(tt.name, func(t *testing.T) {
			t.Parallel()

			var target model.Date
			err := json.Unmarshal([]byte(tt.json), &target)
			if tt.expectedJSONSyntaxError {
				var syntaxError *json.SyntaxError
				assert.ErrorAs(t, err, &syntaxError)
			} else {
				assert.NoError(t, err)
			}
			assert.Equal(t, tt.expectedT, target)
		})
	}
}
