package model

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

type Ennoblement struct {
	ID        int64          `json:"id"`
	Village   VillageMeta    `json:"village"`
	NewOwner  NullPlayerMeta `json:"newOwner" extensions:"x-nullable"`
	Points    int64          `json:"points"`
	CreatedAt time.Time      `json:"createdAt" format:"date-time"`
} // @name Ennoblement

func NewEnnoblement(e domain.EnnoblementWithRelations) Ennoblement {
	return Ennoblement{
		ID: e.ID,
		Village: NewVillageMeta(domain.VillageMetaWithRelations{
			VillageMeta: e.Village,
			Player: domain.NullPlayerMetaWithRelations{
				Valid: e.OldOwner.Valid,
				Player: domain.PlayerMetaWithRelations{
					PlayerMeta: e.OldOwner.Player,
					Tribe:      e.OldTribe,
				},
			},
		}),
		NewOwner: NullPlayerMeta{
			Valid: e.NewOwner.Valid,
			Player: NewPlayerMeta(domain.PlayerMetaWithRelations{
				PlayerMeta: e.NewOwner.Player,
				Tribe:      e.NewTribe,
			}),
		},
		Points:    e.Points,
		CreatedAt: e.CreatedAt,
	}
}

type ListEnnoblementsResp struct {
	Data []Ennoblement `json:"data"`
} // @name ListEnnoblementsResp

func NewListEnnoblementsResp(ennoblements []domain.EnnoblementWithRelations) ListEnnoblementsResp {
	resp := ListEnnoblementsResp{
		Data: make([]Ennoblement, 0, len(ennoblements)),
	}
	for _, e := range ennoblements {
		resp.Data = append(resp.Data, NewEnnoblement(e))
	}
	return resp
}
