package model_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestNewEnnoblement(t *testing.T) {
	t.Parallel()

	ennoblement := domain.EnnoblementWithRelations{
		Ennoblement: domain.Ennoblement{
			ID: 1234,
			BaseEnnoblement: domain.BaseEnnoblement{
				VillageID:  123,
				NewOwnerID: 124,
				NewTribeID: 125,
				OldOwnerID: 126,
				OldTribeID: 127,
				Points:     128,
				CreatedAt:  time.Now(),
			},
			ServerKey: "pl151",
		},
		Village: domain.VillageMeta{
			ID:        123,
			Name:      "village name",
			X:         450,
			Y:         450,
			Continent: "K44",
		},
		NewOwner: domain.NullPlayerMeta{
			Valid: true,
			Player: domain.PlayerMeta{
				ID:   124,
				Name: "new owner name",
			},
		},
		NewTribe: domain.NullTribeMeta{
			Valid: true,
			Tribe: domain.TribeMeta{
				ID:   125,
				Name: "new tribe name",
				Tag:  "new tribe tag",
			},
		},
		OldOwner: domain.NullPlayerMeta{
			Valid: true,
			Player: domain.PlayerMeta{
				ID:   126,
				Name: "old owner name",
			},
		},
		OldTribe: domain.NullTribeMeta{
			Valid: true,
			Tribe: domain.TribeMeta{
				ID:   125,
				Name: "old tribe name",
				Tag:  "old tribe tag",
			},
		},
	}
	assertEnnoblement(t, ennoblement, model.NewEnnoblement(ennoblement))
}

func TestNewListEnnoblementsResp(t *testing.T) {
	t.Parallel()

	ennoblements := []domain.EnnoblementWithRelations{
		{
			Ennoblement: domain.Ennoblement{
				ID: 1234,
				BaseEnnoblement: domain.BaseEnnoblement{
					VillageID:  123,
					NewOwnerID: 124,
					NewTribeID: 125,
					OldOwnerID: 126,
					OldTribeID: 127,
					Points:     128,
					CreatedAt:  time.Now(),
				},
				ServerKey: "pl151",
			},
			Village: domain.VillageMeta{
				ID:        123,
				Name:      "village name",
				X:         450,
				Y:         450,
				Continent: "K44",
			},
			NewOwner: domain.NullPlayerMeta{
				Valid: true,
				Player: domain.PlayerMeta{
					ID:   124,
					Name: "new owner name",
				},
			},
			NewTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:   125,
					Name: "new tribe name",
					Tag:  "new tribe tag",
				},
			},
			OldOwner: domain.NullPlayerMeta{
				Valid: true,
				Player: domain.PlayerMeta{
					ID:   126,
					Name: "old owner name",
				},
			},
			OldTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:   125,
					Name: "old tribe name",
					Tag:  "old tribe tag",
				},
			},
		},
		{
			Ennoblement: domain.Ennoblement{
				ID: 1234,
				BaseEnnoblement: domain.BaseEnnoblement{
					VillageID:  311,
					NewOwnerID: 312,
					NewTribeID: 313,
					OldOwnerID: 0,
					OldTribeID: 0,
					Points:     129,
					CreatedAt:  time.Now(),
				},
				ServerKey: "pl151",
			},
			Village: domain.VillageMeta{
				ID:        311,
				Name:      "village name",
				X:         449,
				Y:         450,
				Continent: "K44",
			},
			NewOwner: domain.NullPlayerMeta{
				Valid: true,
				Player: domain.PlayerMeta{
					ID:   312,
					Name: "new owner name",
				},
			},
			NewTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:   313,
					Name: "new tribe name",
					Tag:  "new tribe tag",
				},
			},
		},
	}
	resp := model.NewListEnnoblementsResp(ennoblements)
	assert.Len(t, resp.Data, len(ennoblements))
	for i, e := range resp.Data {
		assertEnnoblement(t, ennoblements[i], e)
	}
}

func assertEnnoblement(tb testing.TB, de domain.EnnoblementWithRelations, re model.Ennoblement) {
	tb.Helper()

	assert.Equal(tb, de.ID, re.ID)
	assert.Equal(tb, de.Points, re.Points)
	assertVillageMeta(tb, domain.VillageMetaWithRelations{
		VillageMeta: de.Village,
		Player: domain.NullPlayerMetaWithRelations{
			Valid: de.OldOwner.Valid,
			Player: domain.PlayerMetaWithRelations{
				PlayerMeta: de.OldOwner.Player,
				Tribe:      de.OldTribe,
			},
		},
	}, re.Village)
	assertNullPlayerMeta(tb, domain.NullPlayerMetaWithRelations{
		Valid: de.NewOwner.Valid,
		Player: domain.PlayerMetaWithRelations{
			PlayerMeta: de.NewOwner.Player,
			Tribe:      de.NewTribe,
		},
	}, re.NewOwner)
	assert.Equal(tb, de.CreatedAt, re.CreatedAt)
}
