package model

import (
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

type PlayerSnapshot struct {
	ID          int64         `json:"id"`
	NumVillages int64         `json:"numVillages"`
	Points      int64         `json:"points"`
	Rank        int64         `json:"rank"`
	Tribe       NullTribeMeta `json:"tribe" extensions:"x-nullable"`
	RankAtt     int64         `json:"rankAtt"`
	ScoreAtt    int64         `json:"scoreAtt"`
	RankDef     int64         `json:"rankDef"`
	ScoreDef    int64         `json:"scoreDef"`
	RankSup     int64         `json:"rankSup"`
	ScoreSup    int64         `json:"scoreSup"`
	RankTotal   int64         `json:"rankTotal"`
	ScoreTotal  int64         `json:"scoreTotal"`
	Date        Date          `json:"date" swaggertype:"string" format:"date"`
} // @name PlayerSnapshot

func NewPlayerSnapshot(s domain.PlayerSnapshotWithRelations) PlayerSnapshot {
	var t NullTribeMeta
	if s.Tribe.Valid {
		t = NullTribeMeta{
			Valid: true,
			Tribe: NewTribeMeta(s.Tribe.Tribe),
		}
	}

	return PlayerSnapshot{
		ID:          s.ID,
		NumVillages: s.NumVillages,
		Points:      s.Points,
		Rank:        s.Rank,
		Tribe:       t,
		RankAtt:     s.RankAtt,
		ScoreAtt:    s.ScoreAtt,
		RankDef:     s.RankDef,
		ScoreDef:    s.ScoreDef,
		RankSup:     s.RankSup,
		ScoreSup:    s.ScoreSup,
		RankTotal:   s.RankTotal,
		ScoreTotal:  s.ScoreTotal,
		Date:        Date{s.Date},
	}
}

type ListPlayerSnapshotsResp struct {
	Data []PlayerSnapshot `json:"data"`
} // @name ListPlayerSnapshotsResp

func NewListPlayerSnapshotsResp(snapshots []domain.PlayerSnapshotWithRelations) ListPlayerSnapshotsResp {
	resp := ListPlayerSnapshotsResp{
		Data: make([]PlayerSnapshot, 0, len(snapshots)),
	}
	for _, p := range snapshots {
		resp.Data = append(resp.Data, NewPlayerSnapshot(p))
	}
	return resp
}
