package model_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"
)

func TestNewPlayerSnapshot(t *testing.T) {
	t.Parallel()

	snapshot := domain.PlayerSnapshotWithRelations{
		PlayerSnapshot: domain.PlayerSnapshot{
			OpponentsDefeated: domain.OpponentsDefeated{
				RankAtt:    99,
				ScoreAtt:   100,
				RankDef:    101,
				ScoreDef:   102,
				RankSup:    103,
				ScoreSup:   104,
				RankTotal:  105,
				ScoreTotal: 106,
			},
			ID:          999,
			PlayerID:    150,
			ServerKey:   "pl151",
			NumVillages: 100,
			Points:      101,
			Rank:        102,
			TribeID:     15,
			Date:        time.Now(),
			CreatedAt:   time.Now().Add(-5 * time.Minute),
		},
		Tribe: domain.NullTribeMeta{
			Valid: true,
			Tribe: domain.TribeMeta{
				ID:         15,
				Name:       "Name",
				Tag:        "Tag",
				ProfileURL: "profile-15",
			},
		},
	}
	assertPlayerSnapshot(t, snapshot, model.NewPlayerSnapshot(snapshot))
}

func TestNewListPlayerSnapshotsResp(t *testing.T) {
	t.Parallel()

	snapshots := []domain.PlayerSnapshotWithRelations{
		{
			PlayerSnapshot: domain.PlayerSnapshot{
				OpponentsDefeated: domain.OpponentsDefeated{
					RankAtt:    99,
					ScoreAtt:   100,
					RankDef:    101,
					ScoreDef:   102,
					RankSup:    103,
					ScoreSup:   104,
					RankTotal:  105,
					ScoreTotal: 106,
				},
				ID:          999,
				PlayerID:    150,
				ServerKey:   "pl151",
				NumVillages: 100,
				Points:      101,
				Rank:        102,
				TribeID:     15,
				Date:        time.Now(),
				CreatedAt:   time.Now().Add(-5 * time.Minute),
			},
			Tribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:         15,
					Name:       "Name",
					Tag:        "Tag",
					ProfileURL: "profile-15",
				},
			},
		},
		{
			PlayerSnapshot: domain.PlayerSnapshot{
				OpponentsDefeated: domain.OpponentsDefeated{
					RankAtt:    999,
					ScoreAtt:   1000,
					RankDef:    1010,
					ScoreDef:   1020,
					RankSup:    1030,
					ScoreSup:   1040,
					RankTotal:  1050,
					ScoreTotal: 1060,
				},
				ID:          9990,
				PlayerID:    150,
				ServerKey:   "pl151",
				NumVillages: 1000,
				Points:      1010,
				Rank:        1020,
				TribeID:     0,
				Date:        time.Now().Add(-24 * time.Hour),
				CreatedAt:   time.Now().Add(-36 * time.Hour),
			},
			Tribe: domain.NullTribeMeta{
				Valid: false,
			},
		},
	}
	resp := model.NewListPlayerSnapshotsResp(snapshots)
	assert.Len(t, resp.Data, len(snapshots))
	for i, snapshot := range resp.Data {
		assertPlayerSnapshot(t, snapshots[i], snapshot)
	}
}

func assertPlayerSnapshot(tb testing.TB, ds domain.PlayerSnapshotWithRelations, rs model.PlayerSnapshot) {
	tb.Helper()

	assert.Equal(tb, ds.ID, rs.ID)
	assert.Equal(tb, ds.NumVillages, rs.NumVillages)
	assert.Equal(tb, ds.Points, rs.Points)
	assert.Equal(tb, ds.Rank, rs.Rank)
	assert.Equal(tb, ds.RankAtt, rs.RankAtt)
	assert.Equal(tb, ds.ScoreAtt, rs.ScoreAtt)
	assert.Equal(tb, ds.RankDef, rs.RankDef)
	assert.Equal(tb, ds.ScoreDef, rs.ScoreDef)
	assert.Equal(tb, ds.RankSup, rs.RankSup)
	assert.Equal(tb, ds.ScoreSup, rs.ScoreSup)
	assert.Equal(tb, ds.RankTotal, rs.RankTotal)
	assert.Equal(tb, ds.ScoreTotal, rs.ScoreTotal)
	assert.Equal(tb, ds.Date, rs.Date.Time)
	assertNullTribeMeta(tb, ds.Tribe, rs.Tribe)
}
