package model

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

type Server struct {
	Key                      string    `json:"key" example:"pl151"`
	URL                      string    `json:"url" example:"https://pl151.plemiona.pl"`
	Open                     bool      `json:"open"`
	NumPlayers               int64     `json:"numPlayers"`
	NumTribes                int64     `json:"numTribes"`
	NumVillages              int64     `json:"numVillages"`
	NumPlayerVillages        int64     `json:"numPlayerVillages"`
	NumBarbarianVillages     int64     `json:"numBarbarianVillages"`
	NumBonusVillages         int64     `json:"numBonusVillages"`
	CreatedAt                time.Time `json:"createdAt" format:"date-time"`
	PlayerDataUpdatedAt      NullTime  `json:"playerDataUpdatedAt" swaggertype:"string" format:"date-time" extensions:"x-nullable"`
	TribeDataUpdatedAt       NullTime  `json:"tribeDataUpdatedAt" swaggertype:"string" format:"date-time" extensions:"x-nullable"`
	VillageDataUpdatedAt     NullTime  `json:"villageDataUpdatedAt" swaggertype:"string" format:"date-time" extensions:"x-nullable"`
	EnnoblementDataUpdatedAt NullTime  `json:"ennoblementDataUpdatedAt" swaggertype:"string" format:"date-time" extensions:"x-nullable"`
} // @name Server

func NewServer(s domain.Server) Server {
	return Server{
		Key:                  s.Key,
		URL:                  s.URL,
		Open:                 s.Open,
		NumPlayers:           s.NumPlayers,
		NumTribes:            s.NumTribes,
		NumVillages:          s.NumVillages,
		NumPlayerVillages:    s.NumPlayerVillages,
		NumBarbarianVillages: s.NumBarbarianVillages,
		NumBonusVillages:     s.NumBonusVillages,
		CreatedAt:            s.CreatedAt,
		PlayerDataUpdatedAt: NullTime{
			Time:  s.PlayerDataUpdatedAt,
			Valid: !s.PlayerDataUpdatedAt.IsZero(),
		},
		TribeDataUpdatedAt: NullTime{
			Time:  s.TribeDataUpdatedAt,
			Valid: !s.TribeDataUpdatedAt.IsZero(),
		},
		VillageDataUpdatedAt: NullTime{
			Time:  s.VillageDataUpdatedAt,
			Valid: !s.VillageDataUpdatedAt.IsZero(),
		},
		EnnoblementDataUpdatedAt: NullTime{
			Time:  s.EnnoblementDataUpdatedAt,
			Valid: !s.EnnoblementDataUpdatedAt.IsZero(),
		},
	}
}

// ServerMeta represents basic server information
// @Description Basic server information
type ServerMeta struct {
	Key  string `json:"key" example:"pl151"`
	URL  string `json:"url" example:"https://pl151.plemiona.pl"`
	Open bool   `json:"open"`
} // @name ServerMeta

func NewServerMeta(s domain.ServerMeta) ServerMeta {
	return ServerMeta{
		Key:  s.Key,
		URL:  s.URL,
		Open: s.Open,
	}
}

type ListServersResp struct {
	Data []Server `json:"data"`
} // @name ListServersResp

func NewListServersResp(servers []domain.Server) ListServersResp {
	resp := ListServersResp{
		Data: make([]Server, 0, len(servers)),
	}
	for _, srv := range servers {
		resp.Data = append(resp.Data, NewServer(srv))
	}
	return resp
}

type GetServerResp struct {
	Data Server `json:"data"`
} // @name GetServerResp

func NewGetServerResp(v domain.Server) GetServerResp {
	return GetServerResp{
		Data: NewServer(v),
	}
}
