package model_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestNewServer(t *testing.T) {
	t.Parallel()

	srv := domain.Server{
		Key:                  "pl151",
		URL:                  "https://pl151.plemiona.pl",
		Open:                 true,
		Special:              false,
		NumPlayers:           1234,
		NumTribes:            1235,
		NumVillages:          1236,
		NumPlayerVillages:    1237,
		NumBarbarianVillages: 1238,
		NumBonusVillages:     1239,
		Config: domain.ServerConfig{
			Speed: 0.25,
		},
		BuildingInfo: domain.BuildingInfo{
			Main: domain.Building{BuildTime: 0.33334},
		},
		UnitInfo: domain.UnitInfo{
			Axe: domain.Unit{Attack: 123},
		},
		CreatedAt:                time.Now().Add(-20 * time.Hour),
		PlayerDataUpdatedAt:      time.Time{},
		TribeDataUpdatedAt:       time.Now(),
		VillageDataUpdatedAt:     time.Now().Add(-5 * time.Hour),
		EnnoblementDataUpdatedAt: time.Now().Add(-1 * time.Hour),
		VersionCode:              "pl",
	}
	assertServer(t, srv, model.NewServer(srv))
}

func TestNewServerMeta(t *testing.T) {
	t.Parallel()

	srv := domain.ServerMeta{
		Key:  "pl151",
		URL:  "https://pl151.plemiona.pl",
		Open: true,
	}
	assertServerMeta(t, srv, model.NewServerMeta(srv))
}

func TestNewListServersResp(t *testing.T) {
	t.Parallel()

	servers := []domain.Server{
		{
			Key:                  "pl151",
			URL:                  "https://pl151.plemiona.pl",
			Open:                 true,
			Special:              false,
			NumPlayers:           1234,
			NumTribes:            1235,
			NumVillages:          1236,
			NumPlayerVillages:    1237,
			NumBarbarianVillages: 1238,
			NumBonusVillages:     1239,
			Config:               domain.ServerConfig{},
			BuildingInfo: domain.BuildingInfo{
				Main: domain.Building{BuildTime: 0.33334},
			},
			UnitInfo:                 domain.UnitInfo{},
			CreatedAt:                time.Now().Add(-20 * time.Hour),
			PlayerDataUpdatedAt:      time.Time{},
			TribeDataUpdatedAt:       time.Now(),
			VillageDataUpdatedAt:     time.Now().Add(-5 * time.Hour),
			EnnoblementDataUpdatedAt: time.Time{},
			VersionCode:              "pl",
		},
		{
			Key:                  "pl143",
			URL:                  "https://pl143.plemiona.pl",
			Open:                 false,
			Special:              false,
			NumPlayers:           2234,
			NumTribes:            2235,
			NumVillages:          2236,
			NumPlayerVillages:    2237,
			NumBarbarianVillages: 2238,
			NumBonusVillages:     2239,
			Config:               domain.ServerConfig{},
			BuildingInfo:         domain.BuildingInfo{},
			UnitInfo: domain.UnitInfo{
				Axe: domain.Unit{Attack: 123},
			},
			CreatedAt:                time.Now().Add(-40 * time.Hour),
			PlayerDataUpdatedAt:      time.Now().Add(-1 * time.Second),
			TribeDataUpdatedAt:       time.Now().Add(-5 * time.Second),
			VillageDataUpdatedAt:     time.Now(),
			EnnoblementDataUpdatedAt: time.Now().Add(-1 * time.Minute),
			VersionCode:              "pl",
		},
	}
	resp := model.NewListServersResp(servers)
	assert.Len(t, resp.Data, len(servers))
	for i, srv := range resp.Data {
		assertServer(t, servers[i], srv)
	}
}

func TestNewGetServerResp(t *testing.T) {
	t.Parallel()

	srv := domain.Server{
		Key:                  "pl151",
		URL:                  "https://pl151.plemiona.pl",
		Open:                 true,
		Special:              false,
		NumPlayers:           1234,
		NumTribes:            1235,
		NumVillages:          1236,
		NumPlayerVillages:    1237,
		NumBarbarianVillages: 1238,
		NumBonusVillages:     1239,
		Config: domain.ServerConfig{
			Speed: 0.25,
		},
		CreatedAt:                time.Now().Add(-20 * time.Hour),
		PlayerDataUpdatedAt:      time.Now().Add(-115 * time.Minute),
		TribeDataUpdatedAt:       time.Now(),
		VillageDataUpdatedAt:     time.Now().Add(-5 * time.Hour),
		EnnoblementDataUpdatedAt: time.Now().Add(-1 * time.Minute),
		VersionCode:              "pl",
	}
	assertServer(t, srv, model.NewGetServerResp(srv).Data)
}

func assertServer(tb testing.TB, dsrv domain.Server, rsrv model.Server) {
	tb.Helper()

	assert.Equal(tb, dsrv.Key, rsrv.Key)
	assert.Equal(tb, dsrv.URL, rsrv.URL)
	assert.Equal(tb, dsrv.Open, rsrv.Open)
	assert.Equal(tb, dsrv.NumPlayers, rsrv.NumPlayers)
	assert.Equal(tb, dsrv.NumTribes, rsrv.NumTribes)
	assert.Equal(tb, dsrv.NumVillages, rsrv.NumVillages)
	assert.Equal(tb, dsrv.NumPlayerVillages, rsrv.NumPlayerVillages)
	assert.Equal(tb, dsrv.NumBarbarianVillages, rsrv.NumBarbarianVillages)
	assert.Equal(tb, dsrv.NumBonusVillages, rsrv.NumBonusVillages)
	assert.Equal(tb, dsrv.CreatedAt, rsrv.CreatedAt)
	assertNullTime(tb, dsrv.PlayerDataUpdatedAt, rsrv.PlayerDataUpdatedAt)
	assertNullTime(tb, dsrv.TribeDataUpdatedAt, rsrv.TribeDataUpdatedAt)
	assertNullTime(tb, dsrv.VillageDataUpdatedAt, rsrv.VillageDataUpdatedAt)
	assertNullTime(tb, dsrv.EnnoblementDataUpdatedAt, rsrv.EnnoblementDataUpdatedAt)
}

func assertServerMeta(tb testing.TB, dsrv domain.ServerMeta, rsrv model.ServerMeta) {
	tb.Helper()

	assert.Equal(tb, dsrv.Key, rsrv.Key)
	assert.Equal(tb, dsrv.URL, rsrv.URL)
	assert.Equal(tb, dsrv.Open, rsrv.Open)
}
