package model

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

type TribeChange struct {
	ID        int64         `json:"id"`
	Player    PlayerMeta    `json:"player"`
	NewTribe  NullTribeMeta `json:"newTribe"`
	CreatedAt time.Time     `json:"createdAt" format:"date-time"`
} // @name TribeChange

func NewTribeChange(tc domain.TribeChangeWithRelations) TribeChange {
	var newTribe NullTribeMeta
	if tc.NewTribe.Valid {
		newTribe = NullTribeMeta{
			Valid: true,
			Tribe: NewTribeMeta(tc.NewTribe.Tribe),
		}
	}

	return TribeChange{
		ID: tc.ID,
		Player: NewPlayerMeta(domain.PlayerMetaWithRelations{
			PlayerMeta: tc.Player,
			Tribe:      tc.OldTribe,
		}),
		NewTribe:  newTribe,
		CreatedAt: tc.CreatedAt,
	}
}

type ListTribeChangesResp struct {
	Data []TribeChange `json:"data"`
} // @name ListTribeChangesResp

func NewListTribeChangesResp(tcs []domain.TribeChangeWithRelations) ListTribeChangesResp {
	resp := ListTribeChangesResp{
		Data: make([]TribeChange, 0, len(tcs)),
	}
	for _, tc := range tcs {
		resp.Data = append(resp.Data, NewTribeChange(tc))
	}
	return resp
}
