package model_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"
)

func TestNewTribeChange(t *testing.T) {
	t.Parallel()

	tc := domain.TribeChangeWithRelations{
		TribeChange: domain.TribeChange{
			ID:         111,
			PlayerID:   1112,
			NewTribeID: 1113,
			OldTribeID: 1114,
			ServerKey:  "pl151",
			CreatedAt:  time.Now(),
		},
		Player: domain.PlayerMeta{
			ID:         1112,
			Name:       "name-1112",
			ProfileURL: "profile-1112",
		},
		NewTribe: domain.NullTribeMeta{
			Valid: true,
			Tribe: domain.TribeMeta{
				ID:         1113,
				Name:       "name-1113",
				Tag:        "tag-1113",
				ProfileURL: "profile-1113",
			},
		},
		OldTribe: domain.NullTribeMeta{
			Valid: true,
			Tribe: domain.TribeMeta{
				ID:         1114,
				Name:       "name-1114",
				Tag:        "tag-1114",
				ProfileURL: "profile-1114",
			},
		},
	}
	assertTribeChange(t, tc, model.NewTribeChange(tc))
}

func TestNewListTribeChangesResp(t *testing.T) {
	t.Parallel()

	tcs := []domain.TribeChangeWithRelations{
		{
			TribeChange: domain.TribeChange{
				ID:         111,
				PlayerID:   1112,
				NewTribeID: 1113,
				OldTribeID: 1114,
				ServerKey:  "pl151",
				CreatedAt:  time.Now(),
			},
			Player: domain.PlayerMeta{
				ID:         1112,
				Name:       "name-1112",
				ProfileURL: "profile-1112",
			},
			NewTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:         1113,
					Name:       "name-1113",
					Tag:        "tag-1113",
					ProfileURL: "profile-1113",
				},
			},
			OldTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:         1114,
					Name:       "name-1114",
					Tag:        "tag-1114",
					ProfileURL: "profile-1114",
				},
			},
		},
		{
			TribeChange: domain.TribeChange{
				ID:         112,
				PlayerID:   1115,
				NewTribeID: 1120,
				OldTribeID: 0,
				ServerKey:  "pl151",
				CreatedAt:  time.Now(),
			},
			Player: domain.PlayerMeta{
				ID:         1115,
				Name:       "name-1115",
				ProfileURL: "profile-1115",
			},
			NewTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:         1120,
					Name:       "name-1120",
					Tag:        "tag-1120",
					ProfileURL: "profile-1120",
				},
			},
			OldTribe: domain.NullTribeMeta{
				Valid: false,
			},
		},
		{
			TribeChange: domain.TribeChange{
				ID:         113,
				PlayerID:   1120,
				NewTribeID: 0,
				OldTribeID: 1114,
				ServerKey:  "pl151",
				CreatedAt:  time.Now(),
			},
			Player: domain.PlayerMeta{
				ID:         1120,
				Name:       "name-1120",
				ProfileURL: "profile-1120",
			},
			NewTribe: domain.NullTribeMeta{
				Valid: false,
			},
			OldTribe: domain.NullTribeMeta{
				Valid: true,
				Tribe: domain.TribeMeta{
					ID:         1114,
					Name:       "name-1114",
					Tag:        "tag-1114",
					ProfileURL: "profile-1114",
				},
			},
		},
	}
	resp := model.NewListTribeChangesResp(tcs)
	assert.Len(t, resp.Data, len(tcs))
	for i, tc := range resp.Data {
		assertTribeChange(t, tcs[i], tc)
	}
}

func assertTribeChange(tb testing.TB, dtc domain.TribeChangeWithRelations, rtc model.TribeChange) {
	tb.Helper()

	assert.Equal(tb, dtc.ID, rtc.ID)
	assertPlayerMeta(tb, domain.PlayerMetaWithRelations{
		PlayerMeta: dtc.Player,
		Tribe:      dtc.OldTribe,
	}, rtc.Player)
	assertNullTribeMeta(tb, dtc.NewTribe, rtc.NewTribe)
	assert.Equal(tb, dtc.CreatedAt, rtc.CreatedAt)
}
