package model

import (
	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

type TribeSnapshot struct {
	ID          int64   `json:"id"`
	NumMembers  int64   `json:"numMembers"`
	NumVillages int64   `json:"numVillages"`
	Points      int64   `json:"points"`
	AllPoints   int64   `json:"allPoints"`
	Rank        int64   `json:"rank"`
	Dominance   float64 `json:"dominance"`
	RankAtt     int64   `json:"rankAtt"`
	ScoreAtt    int64   `json:"scoreAtt"`
	RankDef     int64   `json:"rankDef"`
	ScoreDef    int64   `json:"scoreDef"`
	RankTotal   int64   `json:"rankTotal"`
	ScoreTotal  int64   `json:"scoreTotal"`
	Date        Date    `json:"date" swaggertype:"string" format:"date"`
} // @name TribeSnapshot

func NewTribeSnapshot(s domain.TribeSnapshot) TribeSnapshot {
	return TribeSnapshot{
		ID:          s.ID,
		NumMembers:  s.NumMembers,
		NumVillages: s.NumVillages,
		Points:      s.Points,
		AllPoints:   s.AllPoints,
		Rank:        s.Rank,
		Dominance:   s.Dominance,
		RankAtt:     s.RankAtt,
		ScoreAtt:    s.ScoreAtt,
		RankDef:     s.RankDef,
		ScoreDef:    s.ScoreDef,
		RankTotal:   s.RankTotal,
		ScoreTotal:  s.ScoreTotal,
		Date:        Date{s.Date},
	}
}

type ListTribeSnapshotsResp struct {
	Data []TribeSnapshot `json:"data"`
} // @name ListTribeSnapshotsResp

func NewListTribeSnapshotsResp(snapshots []domain.TribeSnapshot) ListTribeSnapshotsResp {
	resp := ListTribeSnapshotsResp{
		Data: make([]TribeSnapshot, 0, len(snapshots)),
	}
	for _, p := range snapshots {
		resp.Data = append(resp.Data, NewTribeSnapshot(p))
	}
	return resp
}
