package model_test

import (
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"
)

func TestNewTribeSnapshot(t *testing.T) {
	t.Parallel()

	snapshot := domain.TribeSnapshot{
		OpponentsDefeated: domain.OpponentsDefeated{
			RankAtt:    99,
			ScoreAtt:   100,
			RankDef:    101,
			ScoreDef:   102,
			RankSup:    103,
			ScoreSup:   104,
			RankTotal:  105,
			ScoreTotal: 106,
		},
		ID:          999,
		TribeID:     15,
		ServerKey:   "pl151",
		NumMembers:  113,
		NumVillages: 100,
		Points:      101,
		AllPoints:   1112,
		Rank:        102,
		Dominance:   1.24,
		Date:        time.Now(),
		CreatedAt:   time.Now().Add(-5 * time.Minute),
	}
	assertTribeSnapshot(t, snapshot, model.NewTribeSnapshot(snapshot))
}

func TestNewListTribeSnapshotsResp(t *testing.T) {
	t.Parallel()

	snapshots := []domain.TribeSnapshot{
		{
			OpponentsDefeated: domain.OpponentsDefeated{
				RankAtt:    99,
				ScoreAtt:   100,
				RankDef:    101,
				ScoreDef:   102,
				RankSup:    103,
				ScoreSup:   104,
				RankTotal:  105,
				ScoreTotal: 106,
			},
			ID:          999,
			TribeID:     150,
			ServerKey:   "pl151",
			NumMembers:  1235,
			NumVillages: 100,
			Points:      101,
			AllPoints:   5321,
			Rank:        102,
			Dominance:   5.15,
			Date:        time.Now(),
			CreatedAt:   time.Now().Add(-5 * time.Minute),
		},
		{
			OpponentsDefeated: domain.OpponentsDefeated{
				RankAtt:    999,
				ScoreAtt:   1000,
				RankDef:    1010,
				ScoreDef:   1020,
				RankSup:    1030,
				ScoreSup:   1040,
				RankTotal:  1050,
				ScoreTotal: 1060,
			},
			ID:          9990,
			TribeID:     150,
			ServerKey:   "pl151",
			NumMembers:  123,
			NumVillages: 1000,
			Points:      1010,
			AllPoints:   321,
			Rank:        1020,
			Dominance:   1.15,
			Date:        time.Now().Add(-24 * time.Hour),
			CreatedAt:   time.Now().Add(-36 * time.Hour),
		},
	}
	resp := model.NewListTribeSnapshotsResp(snapshots)
	assert.Len(t, resp.Data, len(snapshots))
	for i, snapshot := range resp.Data {
		assertTribeSnapshot(t, snapshots[i], snapshot)
	}
}

func assertTribeSnapshot(tb testing.TB, ds domain.TribeSnapshot, rs model.TribeSnapshot) {
	tb.Helper()

	assert.Equal(tb, ds.ID, rs.ID)
	assert.Equal(tb, ds.NumVillages, rs.NumVillages)
	assert.Equal(tb, ds.NumMembers, rs.NumMembers)
	assert.Equal(tb, ds.Points, rs.Points)
	assert.Equal(tb, ds.AllPoints, rs.AllPoints)
	assert.Equal(tb, ds.Dominance, rs.Dominance)
	assert.Equal(tb, ds.Rank, rs.Rank)
	assert.Equal(tb, ds.RankAtt, rs.RankAtt)
	assert.Equal(tb, ds.ScoreAtt, rs.ScoreAtt)
	assert.Equal(tb, ds.RankDef, rs.RankDef)
	assert.Equal(tb, ds.ScoreDef, rs.ScoreDef)
	assert.Equal(tb, ds.RankTotal, rs.RankTotal)
	assert.Equal(tb, ds.ScoreTotal, rs.ScoreTotal)
	assert.Equal(tb, ds.Date, rs.Date.Time)
}
