package model_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestNewUnitInfo_NewGetUnitInfoResp(t *testing.T) {
	t.Parallel()

	info := domain.UnitInfo{
		Spear: domain.Unit{
			BuildTime: 5.8777,
		},
		Sword: domain.Unit{
			Pop: 1,
		},
		Axe: domain.Unit{
			Speed: 13.7777,
		},
		Archer: domain.Unit{
			Attack: 10,
		},
		Spy: domain.Unit{
			Defense: 1,
		},
		Light: domain.Unit{
			DefenseCavalry: 15,
		},
		Marcher: domain.Unit{
			DefenseArcher: 10,
		},
		Heavy: domain.Unit{
			BuildTime: 30.1234,
			Pop:       6,
		},
		Ram: domain.Unit{
			Speed:  30.849,
			Attack: 1,
		},
		Catapult: domain.Unit{
			Defense:        12,
			DefenseCavalry: 123,
		},
		Knight: domain.Unit{
			DefenseArcher: 1234,
			Carry:         10,
		},
		Snob: domain.Unit{
			BuildTime: 1234.4,
			Pop:       100,
			Speed:     35,
		},
		Militia: domain.Unit{
			BuildTime:      1,
			Pop:            2,
			Speed:          3,
			Attack:         4,
			Defense:        5,
			DefenseCavalry: 6,
			DefenseArcher:  7,
			Carry:          8,
		},
	}
	assertUnitInfo(t, info, model.NewUnitInfo(info))
	assertUnitInfo(t, info, model.NewGetUnitInfoResp(info).Data)
}

func assertUnitInfo(tb testing.TB, dui domain.UnitInfo, rui model.UnitInfo) {
	tb.Helper()

	assert.EqualValues(tb, dui.Spear, rui.Spear)
	assert.EqualValues(tb, dui.Sword, rui.Sword)
	assert.EqualValues(tb, dui.Axe, rui.Axe)
	assert.EqualValues(tb, dui.Archer, rui.Archer)
	assert.EqualValues(tb, dui.Spy, rui.Spy)
	assert.EqualValues(tb, dui.Light, rui.Light)
	assert.EqualValues(tb, dui.Marcher, rui.Marcher)
	assert.EqualValues(tb, dui.Heavy, rui.Heavy)
	assert.EqualValues(tb, dui.Ram, rui.Ram)
	assert.EqualValues(tb, dui.Catapult, rui.Catapult)
	assert.EqualValues(tb, dui.Knight, rui.Knight)
	assert.EqualValues(tb, dui.Snob, rui.Snob)
	assert.EqualValues(tb, dui.Militia, rui.Militia)
}
