package model_test

import (
	"testing"

	"gitea.dwysokinski.me/twhelp/core/internal/router/rest/internal/model"
	"github.com/stretchr/testify/assert"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

func TestNewVersion(t *testing.T) {
	t.Parallel()

	version := domain.Version{
		Code:     "pl",
		Name:     "Poland",
		Host:     "plemiona.pl",
		Timezone: "Europe/Warsaw",
	}
	assertVersion(t, version, model.NewVersion(version))
}

func TestNewListVersionsResp(t *testing.T) {
	t.Parallel()

	versions := []domain.Version{
		{
			Code:     "pl",
			Name:     "Poland",
			Host:     "plemiona.pl",
			Timezone: "Europe/Warsaw",
		},
		{
			Code:     "uk",
			Name:     "United Kingdom",
			Host:     "tribalwars.co.uk",
			Timezone: "Europe/London",
		},
		{
			Code:     "hu",
			Name:     "Hungary",
			Host:     "klanhaboru.hu",
			Timezone: "Europe/Budapest",
		},
	}

	resp := model.NewListVersionsResp(versions)
	assert.Len(t, resp.Data, len(versions))
	for i, vr := range resp.Data {
		assertVersion(t, versions[i], vr)
	}
}

func TestNewGetVersionResp(t *testing.T) {
	t.Parallel()

	version := domain.Version{
		Code:     "pl",
		Name:     "Poland",
		Host:     "plemiona.pl",
		Timezone: "Europe/Warsaw",
	}
	assertVersion(t, version, model.NewGetVersionResp(version).Data)
}

func assertVersion(tb testing.TB, vd domain.Version, vr model.Version) {
	tb.Helper()

	assert.Equal(tb, vd.Code, vr.Code)
	assert.Equal(tb, vd.Name, vr.Name)
	assert.Equal(tb, vd.Host, vr.Host)
	assert.Equal(tb, vd.Timezone, vr.Timezone)
}
