package model

import (
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
)

type Village struct {
	ID         int64          `json:"id"`
	Name       string         `json:"name" example:"Village"`
	FullName   string         `json:"fullName" example:"Village (450|450) K44"`
	Points     int64          `json:"points"`
	X          int64          `json:"x" example:"450"`
	Y          int64          `json:"y" example:"450"`
	Continent  string         `json:"continent" example:"K44"`
	Bonus      int64          `json:"bonus"`
	Player     NullPlayerMeta `json:"player" extensions:"x-nullable"`
	ProfileURL string         `json:"profileUrl"`
	CreatedAt  time.Time      `json:"createdAt" format:"date-time"`
} // @name Village

func NewVillage(v domain.VillageWithRelations) Village {
	var p NullPlayerMeta
	if v.Player.Valid {
		p = NullPlayerMeta{
			Valid:  true,
			Player: NewPlayerMeta(v.Player.Player),
		}
	}
	return Village{
		ID:         v.ID,
		Name:       v.Name,
		FullName:   v.FullName(),
		Points:     v.Points,
		X:          v.X,
		Y:          v.Y,
		Continent:  v.Continent,
		Bonus:      v.Bonus,
		Player:     p,
		ProfileURL: v.ProfileURL,
		CreatedAt:  v.CreatedAt,
	}
}

// VillageMeta represents basic village information
// @Description Basic village information
type VillageMeta struct {
	ID         int64          `json:"id"`
	FullName   string         `json:"fullName" example:"Village (450|450) K44"`
	X          int64          `json:"x"`
	Y          int64          `json:"y"`
	Player     NullPlayerMeta `json:"player" extensions:"x-nullable"`
	Continent  string         `json:"continent"`
	ProfileURL string         `json:"profileUrl"`
} // @name VillageMeta

func NewVillageMeta(v domain.VillageMetaWithRelations) VillageMeta {
	var player NullPlayerMeta
	if v.Player.Valid {
		player = NullPlayerMeta{
			Valid:  true,
			Player: NewPlayerMeta(v.Player.Player),
		}
	}

	return VillageMeta{
		ID:         v.ID,
		FullName:   v.FullName(),
		X:          v.X,
		Y:          v.Y,
		Continent:  v.Continent,
		ProfileURL: v.ProfileURL,
		Player:     player,
	}
}

type ListVillagesResp struct {
	Data []Village `json:"data"`
} // @name ListVillagesResp

func NewListVillagesResp(villages []domain.VillageWithRelations) ListVillagesResp {
	resp := ListVillagesResp{
		Data: make([]Village, 0, len(villages)),
	}
	for _, v := range villages {
		resp.Data = append(resp.Data, NewVillage(v))
	}
	return resp
}

type GetVillageResp struct {
	Data Village `json:"data"`
} // @name GetVillageResp

func NewGetVillageResp(v domain.VillageWithRelations) GetVillageResp {
	return GetVillageResp{
		Data: NewVillage(v),
	}
}
